@@ -21,6 +21,7 @@ use tor_dirmgr::bridgedesc::BridgeDescMgr;
 use tor_dirmgr::{DirMgrStore, Timeliness};
 use tor_error::{error_report, internal, Bug};
 use tor_guardmgr::{GuardMgr, RetireCircuits};
+use tor_memquota::MemoryQuotaTracker;
 use tor_netdir::{params::NetParameters, NetDirProvider};
 #[cfg(feature = "onion-service-service")]
 use tor_persist::state_dir::StateDirectory;
@@ -37,14 +38,20 @@ use tor_rtcompat::{Runtime, SleepProviderExt};
 use {
     tor_config::BoolOrAuto,
     tor_hsclient::{HsClientConnector, HsClientDescEncKeypairSpecifier, HsClientSecretKeysBuilder},
-    tor_hscrypto::pk::{HsClientDescEncKey, HsClientDescEncKeypair},
+    tor_hscrypto::pk::{HsClientDescEncKey, HsClientDescEncKeypair, HsClientDescEncSecretKey},
     tor_netdir::DirEvent,
 };
 
 #[cfg(all(feature = "onion-service-client", feature = "experimental-api"))]
-use {tor_hscrypto::pk::HsId, tor_keymgr::KeystoreSelector};
+use {
+    tor_hscrypto::pk::HsId, tor_hscrypto::pk::HsIdKeypair, tor_hsservice::HsIdKeypairSpecifier,
+    tor_keymgr::KeystoreSelector,
+};
 
-use tor_keymgr::{ArtiNativeKeystore, KeyMgr, KeyMgrBuilder};
+use tor_keymgr::{config::arti::ArtiKeystoreKind, ArtiNativeKeystore, KeyMgr, KeyMgrBuilder};
+
+#[cfg(feature = "ephemeral-keystore")]
+use tor_keymgr::ArtiEphemeralKeystore;
 
 use futures::lock::Mutex as AsyncMutex;
 use futures::task::SpawnExt;
@@ -90,6 +97,8 @@ pub struct TorClient<R: Runtime> {
     client_isolation: IsolationToken,
     /// Connection preferences.  Starts out as `Default`,  Inherited by our clones.
     connect_prefs: StreamPrefs,
+    /// Memory quota tracker
+    memquota: Arc<MemoryQuotaTracker>,
     /// Channel manager, used by circuits etc.,
     ///
     /// Used directly by client only for reconfiguration.
@@ -212,51 +221,47 @@ impl InertTorClient {
     /// Returns `Ok(None)` if keystore use is disabled.
     fn create_keymgr(config: &TorClientConfig) -> StdResult<Option<Arc<KeyMgr>>, ErrorDetail> {
         let keystore = config.storage.keystore();
-        if keystore.is_enabled() {
-            let (state_dir, _mistrust) = config.state_dir()?;
-            let key_store_dir = state_dir.join("keystore");
-            let permissions = config.storage.permissions();
+        match keystore.primary_kind() {
+            Some(ArtiKeystoreKind::Native) => {
+                let (state_dir, _mistrust) = config.state_dir()?;
+                let key_store_dir = state_dir.join("keystore");
+                let permissions = config.storage.permissions();
 
-            let arti_store =
-                ArtiNativeKeystore::from_path_and_mistrust(&key_store_dir, permissions)?;
-            info!("Using keystore from {key_store_dir:?}");
+                let native_store =
+                    ArtiNativeKeystore::from_path_and_mistrust(&key_store_dir, permissions)?;
+                info!("Using keystore from {key_store_dir:?}");
 
-            // TODO #1106: make the default store configurable
-            let default_store = arti_store;
+                let keymgr = KeyMgrBuilder::default()
+                    .primary_store(Box::new(native_store))
+                    .build()
+                    .map_err(|_| internal!("failed to build keymgr"))?;
 
-            let keymgr = KeyMgrBuilder::default()
-                .default_store(Box::new(default_store))
-                .build()
-                .map_err(|_| internal!("failed to build keymgr"))?;
-
-            // TODO #858: add support for the C Tor key store
-            Ok(Some(Arc::new(keymgr)))
-        } else {
-            info!("Running without a keystore");
-            Ok(None)
+                // TODO #858: add support for the C Tor key store
+                Ok(Some(Arc::new(keymgr)))
+            }
+            #[cfg(feature = "ephemeral-keystore")]
+            Some(ArtiKeystoreKind::Ephemeral) => {
+                // TODO: make the keystore ID somehow configurable
+                let ephemeral_store: ArtiEphemeralKeystore =
+                    ArtiEphemeralKeystore::new("ephemeral".to_string());
+                let keymgr = KeyMgrBuilder::default()
+                    .primary_store(Box::new(ephemeral_store))
+                    .build()
+                    .map_err(|_| internal!("failed to build keymgr"))?;
+                Ok(Some(Arc::new(keymgr)))
+            }
+            None => {
+                info!("Running without a keystore");
+                Ok(None)
+            }
+            ty => Err(internal!("unrecognized keystore type {ty:?}").into()),
         }
     }
 
     /// Generate a service discovery keypair for connecting to a hidden service running in
     /// "restricted discovery" mode.
     ///
-    /// The `selector` argument is used for choosing the keystore in which to generate the keypair.
-    /// While most users will want to write to the [`Default`](KeystoreSelector::Default), if you
-    /// have configured this `TorClient` with a non-default keystore and wish to generate the
-    /// keypair in it, you can do so by calling this function with a [KeystoreSelector::Id]
-    /// specifying the keystore ID of your keystore.
-    ///
-    // Note: the selector argument exists for future-proofing reasons. We don't currently support
-    // configuring custom or non-default keystores (see #1106).
-    ///
-    /// Returns an error if the key already exists in the specified key store.
-    ///
-    /// Important: the public part of the generated keypair must be shared with the service, and
-    /// the service needs to be configured to allow the owner of its private counterpart to
-    /// discover its introduction points. The caller is responsible for sharing the public part of
-    /// the key with the hidden service.
-    ///
-    /// This function does not require the `TorClient` to be running or bootstrapped.
+    /// See [`TorClient::generate_service_discovery_key`].
     //
     // TODO: decide whether this should use get_or_generate before making it
     // non-experimental
@@ -293,11 +298,83 @@ impl InertTorClient {
         Ok(key.public().clone())
     }
 
+    /// Rotate the service discovery keypair for connecting to a hidden service running in
+    /// "restricted discovery" mode.
+    ///
+    /// See [`TorClient::rotate_service_discovery_key`].
+    #[cfg(all(
+        feature = "onion-service-client",
+        feature = "experimental-api",
+        feature = "keymgr"
+    ))]
+    #[cfg_attr(
+        docsrs,
+        doc(cfg(all(
+            feature = "onion-service-client",
+            feature = "experimental-api",
+            feature = "keymgr"
+        )))
+    )]
+    pub fn rotate_service_discovery_key(
+        &self,
+        selector: KeystoreSelector,
+        hsid: HsId,
+    ) -> crate::Result<HsClientDescEncKey> {
+        let mut rng = rand::thread_rng();
+        let spec = HsClientDescEncKeypairSpecifier::new(hsid);
+        let key = self
+            .keymgr
+            .as_ref()
+            .ok_or(ErrorDetail::KeystoreRequired {
+                action: "rotate client service discovery key",
+            })?
+            .generate::<HsClientDescEncKeypair>(
+                &spec, selector, &mut rng, true, /* overwrite */
+            )?;
+
+        Ok(key.public().clone())
+    }
+
+    /// Insert a service discovery secret key for connecting to a hidden service running in
+    /// "restricted discovery" mode
+    ///
+    /// See [`TorClient::insert_service_discovery_key`].
+    #[cfg(all(
+        feature = "onion-service-client",
+        feature = "experimental-api",
+        feature = "keymgr"
+    ))]
+    #[cfg_attr(
+        docsrs,
+        doc(cfg(all(
+            feature = "onion-service-client",
+            feature = "experimental-api",
+            feature = "keymgr"
+        )))
+    )]
+    pub fn insert_service_discovery_key(
+        &self,
+        selector: KeystoreSelector,
+        hsid: HsId,
+        hs_client_desc_enc_secret_key: HsClientDescEncSecretKey,
+    ) -> crate::Result<HsClientDescEncKey> {
+        let spec = HsClientDescEncKeypairSpecifier::new(hsid);
+        let client_desc_enc_key = HsClientDescEncKey::from(&hs_client_desc_enc_secret_key);
+        let client_desc_enc_keypair =
+            HsClientDescEncKeypair::new(client_desc_enc_key.clone(), hs_client_desc_enc_secret_key);
+        let _key = self
+            .keymgr
+            .as_ref()
+            .ok_or(ErrorDetail::KeystoreRequired {
+                action: "insert client service discovery key",
+            })?
+            .insert::<HsClientDescEncKeypair>(client_desc_enc_keypair, &spec, selector, false)?;
+        Ok(client_desc_enc_key)
+    }
+
     /// Return the service discovery public key for the service with the specified `hsid`.
     ///
-    /// Returns `Ok(None)` if no such key exists.
-    ///
-    /// This function does not require the `TorClient` to be running or bootstrapped.
+    /// See [`TorClient::get_service_discovery_key`].
     #[cfg(all(feature = "onion-service-client", feature = "experimental-api"))]
     #[cfg_attr(
         docsrs,
@@ -319,6 +396,41 @@ impl InertTorClient {
 
         Ok(key)
     }
+
+    /// Removes the service discovery keypair for the service with the specified `hsid`.
+    ///
+    /// See [`TorClient::remove_service_discovery_key`].
+    #[cfg(all(
+        feature = "onion-service-client",
+        feature = "experimental-api",
+        feature = "keymgr"
+    ))]
+    #[cfg_attr(
+        docsrs,
+        doc(cfg(all(
+            feature = "onion-service-client",
+            feature = "experimental-api",
+            feature = "keymgr"
+        )))
+    )]
+    pub fn remove_service_discovery_key(
+        &self,
+        selector: KeystoreSelector,
+        hsid: HsId,
+    ) -> crate::Result<Option<()>> {
+        let spec = HsClientDescEncKeypairSpecifier::new(hsid);
+        let result = self
+            .keymgr
+            .as_ref()
+            .ok_or(ErrorDetail::KeystoreRequired {
+                action: "remove client service discovery key",
+            })?
+            .remove::<HsClientDescEncKeypair>(&spec, selector)?;
+        match result {
+            Some(_) => Ok(Some(())),
+            None => Ok(None),
+        }
+    }
 }
 
 /// Preferences for whether a [`TorClient`] should bootstrap on its own or not.
@@ -686,6 +798,8 @@ impl<R: Runtime> TorClient<R> {
             .into());
         }
 
+        let memquota = MemoryQuotaTracker::new(&runtime, config.system.memory.clone())?;
+
         let (state_dir, mistrust) = config.state_dir()?;
 
         let dormant = DormantMode::Normal;
@@ -731,14 +845,16 @@ impl<R: Runtime> TorClient<R> {
             mgr
         };
 
-        let circmgr = tor_circmgr::CircMgr::new(
-            config,
-            statemgr.clone(),
-            &runtime,
-            Arc::clone(&chanmgr),
-            guardmgr.clone(),
-        )
-        .map_err(ErrorDetail::CircMgrSetup)?;
+        let circmgr = Arc::new(
+            tor_circmgr::CircMgr::new(
+                config,
+                statemgr.clone(),
+                &runtime,
+                Arc::clone(&chanmgr),
+                guardmgr.clone(),
+            )
+            .map_err(ErrorDetail::CircMgrSetup)?,
+        );
 
         let timeout_cfg = config.stream_timeouts.clone();
 
@@ -771,7 +887,7 @@ impl<R: Runtime> TorClient<R> {
 
         #[cfg(any(feature = "onion-service-client", feature = "onion-service-service"))]
         let hs_circ_pool = {
-            let circpool = tor_circmgr::hspool::HsCircPool::new(&circmgr);
+            let circpool = Arc::new(tor_circmgr::hspool::HsCircPool::new(&circmgr));
             circpool
                 .launch_background_tasks(&runtime, &dirmgr.clone().upcast_arc())
                 .map_err(ErrorDetail::CircMgrSetup)?;
@@ -823,6 +939,7 @@ impl<R: Runtime> TorClient<R> {
             runtime,
             client_isolation,
             connect_prefs: Default::default(),
+            memquota,
             chanmgr,
             circmgr,
             dirmgr_store,
@@ -1038,6 +1155,10 @@ impl<R: Runtime> TorClient<R> {
             how.cannot_change("storage.state_dir").map_err(wrap_err)?;
         }
 
+        self.memquota
+            .reconfigure(new_config.system.memory.clone(), how)
+            .map_err(wrap_err)?;
+
         let retire_circuits = self
             .circmgr
             .reconfigure(new_config, how)
@@ -1532,11 +1653,52 @@ impl<R: Runtime> TorClient<R> {
         Ok((service, stream))
     }
 
+    /// Try to launch an onion service with a given configuration and provided
+    /// [`HsIdKeypair`]. If an onion service with the given nickname already has an
+    /// associated `HsIdKeypair`  in this `TorClient`'s `KeyMgr`, then this operation
+    /// fails rather than overwriting the existing key.
+    ///
+    /// This onion service will not actually handle any requests on its own: you
+    /// will need to
+    /// pull [`RendRequest`](tor_hsservice::RendRequest) objects from the returned stream,
+    /// [`accept`](tor_hsservice::RendRequest::accept) the ones that you want to
+    /// answer, and then wait for them to give you [`StreamRequest`](tor_hsservice::StreamRequest)s.
+    ///
+    /// You may find the [`tor_hsservice::handle_rend_requests`] API helpful for
+    /// translating `RendRequest`s into `StreamRequest`s.
+    ///
+    /// If you want to forward all the requests from an onion service to a set
+    /// of local ports, you may want to use the `tor-hsrproxy` crate.
+    #[cfg(all(feature = "onion-service-service", feature = "experimental-api"))]
+    pub fn launch_onion_service_with_hsid(
+        &self,
+        config: tor_hsservice::OnionServiceConfig,
+        id_keypair: HsIdKeypair,
+    ) -> crate::Result<(
+        Arc<tor_hsservice::RunningOnionService>,
+        impl futures::Stream<Item = tor_hsservice::RendRequest>,
+    )> {
+        let nickname = config.nickname();
+        let hsid_spec = HsIdKeypairSpecifier::new(nickname.clone());
+        let selector = KeystoreSelector::Primary;
+
+        let _kp = self
+            .inert_client
+            .keymgr
+            .as_ref()
+            .ok_or(ErrorDetail::KeystoreRequired {
+                action: "launch onion service ex",
+            })?
+            .insert::<HsIdKeypair>(id_keypair, &hsid_spec, selector, false)?;
+
+        self.launch_onion_service(config)
+    }
+
     /// Generate a service discovery keypair for connecting to a hidden service running in
     /// "restricted discovery" mode.
     ///
     /// The `selector` argument is used for choosing the keystore in which to generate the keypair.
-    /// While most users will want to write to the [`Default`](KeystoreSelector::Default), if you
+    /// While most users will want to write to the [`Primary`](KeystoreSelector::Primary), if you
     /// have configured this `TorClient` with a non-default keystore and wish to generate the
     /// keypair in it, you can do so by calling this function with a [KeystoreSelector::Id]
     /// specifying the keystore ID of your keystore.
@@ -1577,6 +1739,94 @@ impl<R: Runtime> TorClient<R> {
             .generate_service_discovery_key(selector, hsid)
     }
 
+    /// Rotate the service discovery keypair for connecting to a hidden service running in
+    /// "restricted discovery" mode.
+    ///
+    /// **If the specified keystore already contains a restricted discovery keypair
+    /// for the service, it will be overwritten.** Otherwise, a new keypair is generated.
+    ///
+    /// The `selector` argument is used for choosing the keystore in which to generate the keypair.
+    /// While most users will want to write to the [`Primary`](KeystoreSelector::Primary), if you
+    /// have configured this `TorClient` with a non-default keystore and wish to generate the
+    /// keypair in it, you can do so by calling this function with a [KeystoreSelector::Id]
+    /// specifying the keystore ID of your keystore.
+    ///
+    // Note: the selector argument exists for future-proofing reasons. We don't currently support
+    // configuring custom or non-default keystores (see #1106).
+    ///
+    /// Important: the public part of the generated keypair must be shared with the service, and
+    /// the service needs to be configured to allow the owner of its private counterpart to
+    /// discover its introduction points. The caller is responsible for sharing the public part of
+    /// the key with the hidden service.
+    ///
+    /// This function does not require the `TorClient` to be running or bootstrapped.
+    #[cfg(all(
+        feature = "onion-service-client",
+        feature = "experimental-api",
+        feature = "keymgr"
+    ))]
+    #[cfg_attr(
+        docsrs,
+        doc(cfg(all(
+            feature = "onion-service-client",
+            feature = "experimental-api",
+            feature = "keymgr"
+        )))
+    )]
+    pub fn rotate_service_discovery_key(
+        &self,
+        selector: KeystoreSelector,
+        hsid: HsId,
+    ) -> crate::Result<HsClientDescEncKey> {
+        self.inert_client
+            .rotate_service_discovery_key(selector, hsid)
+    }
+
+    /// Insert a service discovery secret key for connecting to a hidden service running in
+    /// "restricted discovery" mode
+    ///
+    /// The `selector` argument is used for choosing the keystore in which to generate the keypair.
+    /// While most users will want to write to the [`Primary`](KeystoreSelector::Primary), if you
+    /// have configured this `TorClient` with a non-default keystore and wish to insert the
+    /// key in it, you can do so by calling this function with a [KeystoreSelector::Id]
+    ///
+    // Note: the selector argument exists for future-proofing reasons. We don't currently support
+    // configuring custom or non-default keystores (see #1106).
+    ///
+    /// Returns an error if the key already exists in the specified key store.
+    ///
+    /// Important: the public part of the generated keypair must be shared with the service, and
+    /// the service needs to be configured to allow the owner of its private counterpart to
+    /// discover its introduction points. The caller is responsible for sharing the public part of
+    /// the key with the hidden service.
+    ///
+    /// This function does not require the `TorClient` to be running or bootstrapped.
+    #[cfg(all(
+        feature = "onion-service-client",
+        feature = "experimental-api",
+        feature = "keymgr"
+    ))]
+    #[cfg_attr(
+        docsrs,
+        doc(cfg(all(
+            feature = "onion-service-client",
+            feature = "experimental-api",
+            feature = "keymgr"
+        )))
+    )]
+    pub fn insert_service_discovery_key(
+        &self,
+        selector: KeystoreSelector,
+        hsid: HsId,
+        hs_client_desc_enc_secret_key: HsClientDescEncSecretKey,
+    ) -> crate::Result<HsClientDescEncKey> {
+        self.inert_client.insert_service_discovery_key(
+            selector,
+            hsid,
+            hs_client_desc_enc_secret_key,
+        )
+    }
+
     /// Return the service discovery public key for the service with the specified `hsid`.
     ///
     /// Returns `Ok(None)` if no such key exists.
@@ -1594,6 +1844,36 @@ impl<R: Runtime> TorClient<R> {
         self.inert_client.get_service_discovery_key(hsid)
     }
 
+    /// Removes the service discovery keypair for the service with the specified `hsid`.
+    ///
+    /// Returns an error if the selected keystore is not the default keystore or one of the
+    /// configured secondary stores.
+    ///
+    /// Returns `Ok(None)` if no such keypair exists whereas `Ok(Some()) means the keypair was successfully removed.
+    ///
+    /// Returns `Err` if an error occurred while trying to remove the key.
+    #[cfg(all(
+        feature = "onion-service-client",
+        feature = "experimental-api",
+        feature = "keymgr"
+    ))]
+    #[cfg_attr(
+        docsrs,
+        doc(cfg(all(
+            feature = "onion-service-client",
+            feature = "experimental-api",
+            feature = "keymgr"
+        )))
+    )]
+    pub fn remove_service_discovery_key(
+        &self,
+        selector: KeystoreSelector,
+        hsid: HsId,
+    ) -> crate::Result<Option<()>> {
+        self.inert_client
+            .remove_service_discovery_key(selector, hsid)
+    }
+
     /// Create (but do not launch) a new
     /// [`OnionService`](tor_hsservice::OnionService)
     /// using the given configuration.
