@@ -29,7 +29,7 @@ pub use tor_guardmgr::bridge::BridgeConfigBuilder;
 pub use tor_guardmgr::bridge::BridgeParseError;
 
 use tor_guardmgr::bridge::BridgeConfig;
-use tor_keymgr::config::arti::{ArtiNativeKeystoreConfig, ArtiNativeKeystoreConfigBuilder};
+use tor_keymgr::config::arti::{ArtiKeystoreConfig, ArtiKeystoreConfigBuilder};
 
 /// Types for configuring how Tor circuits are built.
 pub mod circ {
@@ -187,7 +187,7 @@ pub struct StorageConfig {
     #[cfg(feature = "keymgr")]
     #[builder(sub_builder)]
     #[builder_field_attr(serde(default))]
-    keystore: ArtiNativeKeystoreConfig,
+    keystore: ArtiKeystoreConfig,
 
     /// Configuration about which permissions we want to enforce on our files.
     #[builder(sub_builder(fn_name = "build_for_arti"))]
@@ -231,7 +231,7 @@ impl StorageConfig {
     }
     /// Return the keystore config
     #[allow(clippy::unnecessary_wraps)]
-    pub(crate) fn keystore(&self) -> ArtiNativeKeystoreConfig {
+    pub(crate) fn keystore(&self) -> ArtiKeystoreConfig {
         cfg_if::cfg_if! {
             if #[cfg(feature="keymgr")] {
                 self.keystore.clone()
@@ -566,7 +566,7 @@ pub struct TorClientConfig {
         sub_builder,
         field(
             type = "HashMap<String, i32>",
-            build = "convert_override_net_params(&self.override_net_params)"
+            build = "default_extend(self.override_net_params.clone())"
         )
     )]
     #[builder_field_attr(serde(default))]
@@ -582,6 +582,15 @@ pub struct TorClientConfig {
     #[builder_field_attr(serde(default))]
     pub(crate) channel: ChannelConfig,
 
+    /// Configuration for system resources used by Arti
+    ///
+    /// Note that there are other settings in this section,
+    /// in `arti::cfg::SystemConfig` -
+    /// these two structs overlay here.
+    #[builder(sub_builder)]
+    #[builder_field_attr(serde(default))]
+    pub(crate) system: SystemConfig,
+
     /// Information about how to build paths through the network.
     #[as_ref]
     #[builder(sub_builder)]
@@ -621,17 +630,31 @@ impl tor_config::load::TopLevel for TorClientConfig {
     type Builder = TorClientConfigBuilder;
 }
 
-/// Helper to convert convert_override_net_params
-fn convert_override_net_params(
-    builder: &HashMap<String, i32>,
-) -> tor_netdoc::doc::netstatus::NetParams<i32> {
-    let mut override_net_params = tor_netdoc::doc::netstatus::NetParams::new();
-    for (k, v) in builder {
-        override_net_params.set(k.clone(), *v);
-    }
-    override_net_params
+/// Helper to add overrides to a default collection.
+fn default_extend<T: Default + Extend<X>, X>(to_add: impl IntoIterator<Item = X>) -> T {
+    let mut collection = T::default();
+    collection.extend(to_add);
+    collection
 }
 
+/// Configuration for system resources used by Tor.
+///
+/// You cannot change this section on a running Arti client.
+///
+/// Note that there are other settings in this section,
+/// in `arti_client::config::SystemConfig`.
+#[derive(Debug, Clone, Builder, Eq, PartialEq)]
+#[builder(build_fn(error = "ConfigBuildError"))]
+#[builder(derive(Debug, Serialize, Deserialize))]
+#[non_exhaustive]
+pub struct SystemConfig {
+    /// Memory limits (approximate)
+    #[builder(sub_builder(fn_name = "build"))]
+    #[builder_field_attr(serde(default))]
+    pub(crate) memory: tor_memquota::Config,
+}
+impl_standard_builder! { SystemConfig }
+
 impl tor_circmgr::CircMgrConfig for TorClientConfig {
     #[cfg(all(
         feature = "vanguards",
@@ -719,7 +742,7 @@ impl TorClientConfig {
     }
 
     /// Return the keystore config
-    pub fn keystore(&self) -> ArtiNativeKeystoreConfig {
+    pub fn keystore(&self) -> ArtiKeystoreConfig {
         self.storage.keystore()
     }
 
@@ -734,6 +757,15 @@ impl TorClientConfig {
 
         Ok((state_dir, mistrust))
     }
+
+    /// Access the `tor_memquota` configuration
+    ///
+    /// Ad-hoc accessor for testing purposes.
+    /// (ideally we'd use `visibility` to make fields `pub`, but that doesn't work.)
+    #[cfg(feature = "testing")]
+    pub fn system_memory(&self) -> &tor_memquota::Config {
+        &self.system.memory
+    }
 }
 
 impl TorClientConfigBuilder {
