@@ -110,7 +110,7 @@ pub_if_error_detail! {
 /// The details of this enumeration are not stable: using the `error_detail`
 /// feature will void your semver guarantee.
 ///
-/// Instead of looking at the type, you try to should use the
+/// Instead of looking at the type, you should try to use the
 /// [`kind`](`tor_error::HasKind::kind`) trait method to distinguish among
 /// different kinds of [`Error`](struct@crate::Error).  If that doesn't provide enough information
 /// for your use case, please let us know.
@@ -120,6 +120,10 @@ pub_if_error_detail! {
 #[derive(Error, Clone, Debug)]
 #[non_exhaustive]
 enum ErrorDetail {
+    /// Error setting up the memory quota tracker
+    #[error("Error setting up the memory quota tracker")]
+    MemQuotaSetup(#[from] tor_memquota::StartupError),
+
     /// Error setting up the channel manager
     // TODO: should "chanmgr setup error" be its own type in tor-chanmgr
     #[error("Error setting up the channel manager")]
@@ -387,6 +391,7 @@ impl tor_error::HasKind for ErrorDetail {
             E::ObtainHsCircuit { cause, .. } => cause.kind(),
             E::ExitTimeout => EK::RemoteNetworkTimeout,
             E::BootstrapRequired { .. } => EK::BootstrapRequired,
+            E::MemQuotaSetup(e) => e.kind(),
             E::GuardMgrSetup(e) => e.kind(),
             #[cfg(all(
                 feature = "vanguards",
