@@ -53,7 +53,7 @@ pub mod status;
 
 pub use address::{DangerouslyIntoTorAddr, IntoTorAddr, TorAddr, TorAddrError};
 pub use builder::{TorClientBuilder, MAX_LOCAL_RESOURCE_TIMEOUT};
-pub use client::{BootstrapBehavior, DormantMode, StreamPrefs, TorClient};
+pub use client::{BootstrapBehavior, DormantMode, InertTorClient, StreamPrefs, TorClient};
 pub use config::TorClientConfig;
 
 pub use tor_circmgr::isolation;
