@@ -1,16 +1,18 @@
 //! Authentication for RpcConn.
 
+use std::sync::Arc;
+
 use serde::{de::DeserializeOwned, Deserialize, Serialize};
 
 use crate::msgs::{request::Request, ObjectId};
 
-use super::{ConnectError, RpcConn};
+use super::{ConnectError, ProtoError, RpcConn};
 
 impl super::SuccessResponse {
     /// Try to decode the "result" field of a SuccessResponse as an instance of `D`.
     //
-    // TODO RPC: This might want to be moved and made public.  If we do, it needs a different error type.
-    pub(crate) fn deserialize_as<D: DeserializeOwned>(&self) -> Result<D, ConnectError> {
+    // TODO RPC: Move this.  Possibly, make it public.  If we do, it will need a different error type.
+    pub(crate) fn deserialize_as<D: DeserializeOwned>(&self) -> Result<D, ProtoError> {
         /// Helper object for decoding the "result" field.
         #[derive(Deserialize)]
         struct Response<R> {
@@ -18,7 +20,8 @@ impl super::SuccessResponse {
             result: R,
         }
 
-        let r: Response<D> = serde_json::from_str(self.as_ref())?;
+        let r: Response<D> = serde_json::from_str(self.as_ref())
+            .map_err(|e| ProtoError::CouldNotDecode(Arc::new(e)))?;
         Ok(r.result)
     }
 }
