@@ -94,7 +94,6 @@ mod rpc;
 
 use std::ffi::OsString;
 use std::fmt::Write;
-use std::sync::Arc;
 
 pub use cfg::{
     ApplicationConfig, ApplicationConfigBuilder, ArtiCombinedConfig, ArtiConfig, ArtiConfigBuilder,
@@ -103,26 +102,23 @@ pub use cfg::{
 pub use logging::{LoggingConfig, LoggingConfigBuilder};
 
 use arti_client::config::default_config_files;
-use arti_client::{TorClient, TorClientConfig};
+use arti_client::TorClient;
 use safelog::with_safe_logging_suppressed;
 use tor_config::mistrust::BuilderExt as _;
-use tor_config::{ConfigurationSources, Listen};
-use tor_rtcompat::{BlockOn, Runtime};
+use tor_config::ConfigurationSources;
+use tor_rtcompat::Runtime;
 
 use anyhow::{Context, Error, Result};
 use clap::{value_parser, Arg, ArgAction, Command};
 #[allow(unused_imports)]
 use tracing::{error, info, warn};
 
-#[cfg(all(
-    feature = "onion-service-client",
-    feature = "experimental-api",
-    feature = "keymgr"
-))]
+#[cfg(any(feature = "hsc", feature = "onion-service-service", feature = "relay"))]
 use clap::Subcommand as _;
 
-/// Shorthand for a boxed and pinned Future.
-type PinnedFuture<T> = std::pin::Pin<Box<dyn futures::Future<Output = T>>>;
+#[cfg(feature = "experimental-api")]
+#[cfg_attr(docsrs, doc(cfg(feature = "experimental-api")))]
+pub use subcommands::proxy::run_proxy as run;
 
 /// Create a runtime for Arti to use.
 fn create_runtime() -> std::io::Result<impl Runtime> {
@@ -169,156 +165,6 @@ fn list_enabled_features() -> &'static [&'static str] {
     ]
 }
 
-/// Run the main loop of the proxy.
-///
-/// # Panics
-///
-/// Currently, might panic if things go badly enough wrong
-///
-// TODO: move this to a new subcommands::proxy module.
-#[cfg_attr(feature = "experimental-api", visibility::make(pub))]
-#[cfg_attr(docsrs, doc(cfg(feature = "experimental-api")))]
-async fn run<R: Runtime>(
-    runtime: R,
-    socks_listen: Listen,
-    dns_listen: Listen,
-    config_sources: ConfigurationSources,
-    arti_config: ArtiConfig,
-    client_config: TorClientConfig,
-) -> Result<()> {
-    // Using OnDemand arranges that, while we are bootstrapping, incoming connections wait
-    // for bootstrap to complete, rather than getting errors.
-    use arti_client::BootstrapBehavior::OnDemand;
-    use futures::FutureExt;
-
-    #[cfg(feature = "rpc")]
-    let rpc_path = {
-        if let Some(path) = &arti_config.rpc().rpc_listen {
-            let path = path.path()?;
-            let parent = path
-                .parent()
-                .ok_or(anyhow::anyhow!("No parent directory for rpc_listen path?"))?;
-            client_config
-                .fs_mistrust()
-                .verifier()
-                .make_secure_dir(parent)?;
-            // It's just a unix thing; if we leave this sitting around, binding to it won't
-            // work right.  There is probably a better solution.
-            if path.try_exists()? {
-                std::fs::remove_file(&path)?;
-            }
-
-            Some(path)
-        } else {
-            None
-        }
-    };
-
-    let client_builder = TorClient::with_runtime(runtime.clone())
-        .config(client_config)
-        .bootstrap_behavior(OnDemand);
-    let client = client_builder.create_unbootstrapped_async().await?;
-
-    #[allow(unused_mut)]
-    let mut reconfigurable_modules: Vec<Arc<dyn reload_cfg::ReconfigurableModule>> = vec![
-        Arc::new(client.clone()),
-        Arc::new(reload_cfg::Application::new(arti_config.clone())),
-    ];
-
-    #[cfg(feature = "onion-service-service")]
-    {
-        let onion_services =
-            onion_proxy::ProxySet::launch_new(&client, arti_config.onion_services.clone())?;
-        reconfigurable_modules.push(Arc::new(onion_services));
-    }
-
-    // We weak references here to prevent the thread spawned by watch_for_config_changes from
-    // keeping these modules alive after this function exits.
-    //
-    // NOTE: reconfigurable_modules stores the only strong references to these modules,
-    // so we must keep the variable alive until the end of the function
-    let weak_modules = reconfigurable_modules.iter().map(Arc::downgrade).collect();
-    reload_cfg::watch_for_config_changes(
-        client.runtime(),
-        config_sources,
-        &arti_config,
-        weak_modules,
-    )?;
-
-    #[cfg(all(feature = "rpc", feature = "tokio"))]
-    let rpc_data = {
-        // TODO RPC This code doesn't really belong here; it's just an example.
-        if let Some(listen_path) = rpc_path {
-            let (rpc_state, rpc_state_sender) = rpc::RpcVisibleArtiState::new();
-            // TODO Conceivably this listener belongs on a renamed "proxy" list.
-            let rpc_mgr =
-                rpc::launch_rpc_listener(&runtime, listen_path, client.clone(), rpc_state)?;
-            Some((rpc_mgr, rpc_state_sender))
-        } else {
-            None
-        }
-    };
-
-    let mut proxy: Vec<PinnedFuture<(Result<()>, &str)>> = Vec::new();
-    if !socks_listen.is_empty() {
-        let runtime = runtime.clone();
-        let client = client.isolated_client();
-        proxy.push(Box::pin(async move {
-            let res = socks::run_socks_proxy(
-                runtime,
-                client,
-                socks_listen,
-                #[cfg(all(feature = "rpc", feature = "tokio"))]
-                rpc_data,
-            )
-            .await;
-            (res, "SOCKS")
-        }));
-    }
-
-    #[cfg(feature = "dns-proxy")]
-    if !dns_listen.is_empty() {
-        let runtime = runtime.clone();
-        let client = client.isolated_client();
-        proxy.push(Box::pin(async move {
-            let res = dns::run_dns_resolver(runtime, client, dns_listen).await;
-            (res, "DNS")
-        }));
-    }
-
-    #[cfg(not(feature = "dns-proxy"))]
-    if !dns_listen.is_empty() {
-        warn!(
-            "Tried to specify a DNS proxy address, but Arti was built without dns-proxy support."
-        );
-        return Ok(());
-    }
-
-    if proxy.is_empty() {
-        warn!("No proxy port set; specify -p PORT (for `socks_port`) or -d PORT (for `dns_port`). Alternatively, use the `socks_port` or `dns_port` configuration option.");
-        return Ok(());
-    }
-
-    let proxy = futures::future::select_all(proxy).map(|(finished, _index, _others)| finished);
-    futures::select!(
-        r = exit::wait_for_ctrl_c().fuse()
-            => r.context("waiting for termination signal"),
-        r = proxy.fuse()
-            => r.0.context(format!("{} proxy failure", r.1)),
-        r = async {
-            client.bootstrap().await?;
-            info!("Sufficiently bootstrapped; system SOCKS now functional.");
-            futures::future::pending::<Result<()>>().await
-        }.fuse()
-            => r.context("bootstrap"),
-    )?;
-
-    // The modules can be dropped now, because we are exiting.
-    drop(reconfigurable_modules);
-
-    Ok(())
-}
-
 /// Inner function, to handle a set of CLI arguments and return a single
 /// `Result<()>` for convenient handling.
 ///
@@ -439,47 +285,20 @@ where
     // `maint/check-cli-help`, to the function `help_arg`.
 
     cfg_if::cfg_if! {
-        // TODO (#1250): test the CLI.
         if #[cfg(feature = "onion-service-service")] {
-            let clap_app = clap_app.subcommand(
-                Command::new("hss")
-                    .about(
-                        "Run state management commands for an Arti hidden service",
-                    )
-                    .arg(
-                        Arg::new("nickname")
-                            .short('n')
-                            .long("nickname")
-                            .action(ArgAction::Set)
-                            .value_name("HS_NICKNAME")
-                            .required(true)
-                            .help("The nickname of the service")
-                    )
-                    .subcommand_required(true)
-                    .arg_required_else_help(true)
-                    .subcommand(
-                        Command::new("onion-name")
-                            .about(
-                                "Print the .onion address of a hidden service",
-                            )
-                    )
-            );
+            let clap_app = subcommands::hss::HssSubcommands::augment_subcommands(clap_app);
         }
     }
 
     cfg_if::cfg_if! {
-        if #[cfg(all(feature = "onion-service-client", feature = "experimental-api", feature = "keymgr"))] {
+        if #[cfg(feature = "hsc")] {
             let clap_app = subcommands::hsc::HscSubcommands::augment_subcommands(clap_app);
         }
     }
 
-    // Relay subcommand
     cfg_if::cfg_if! {
         if #[cfg(feature = "relay")] {
-            let clap_app = clap_app.subcommand(
-                Command::new("relay")
-                    .about("Run Arti in relay mode acting as a relay of the Tor network")
-            );
+            let clap_app = subcommands::relay::RelaySubcommands::augment_subcommands(clap_app);
         }
     }
 
@@ -567,39 +386,10 @@ where
         }
     }
 
-    // Match the non optional subcommands that is the one that are always built in.
+    // Check for the "proxy" subcommand.
     if let Some(proxy_matches) = matches.subcommand_matches("proxy") {
-        // Override configured SOCKS and DNS listen addresses from the command line.
-        // This implies listening on localhost ports.
-        let socks_listen = match proxy_matches.get_one::<String>("socks-port") {
-            Some(p) => Listen::new_localhost(p.parse().expect("Invalid port specified")),
-            None => config.proxy().socks_listen.clone(),
-        };
-
-        let dns_listen = match proxy_matches.get_one::<String>("dns-port") {
-            Some(p) => Listen::new_localhost(p.parse().expect("Invalid port specified")),
-            None => config.proxy().dns_listen.clone(),
-        };
-
-        info!(
-            "Starting Arti {} in SOCKS proxy mode on {} ...",
-            env!("CARGO_PKG_VERSION"),
-            socks_listen
-        );
-
-        process::use_max_file_limit(&config);
-
-        let rt_copy = runtime.clone();
-        rt_copy.block_on(run(
-            runtime,
-            socks_listen,
-            dns_listen,
-            cfg_sources,
-            config,
-            client_config,
-        ))?;
-        return Ok(());
-    };
+        return subcommands::proxy::run(runtime, proxy_matches, cfg_sources, config, client_config);
+    }
 
     // Check for the optional "hss" subcommand.
     cfg_if::cfg_if! {
@@ -612,7 +402,7 @@ where
 
     // Check for the optional "hsc" subcommand.
     cfg_if::cfg_if! {
-        if #[cfg(all(feature = "onion-service-client", feature = "experimental-api", feature = "keymgr"))] {
+        if #[cfg(feature = "hsc")] {
             if let Some(hsc_matches) = matches.subcommand_matches("hsc") {
                 return subcommands::hsc::run(runtime, hsc_matches, &client_config);
             }
@@ -622,9 +412,8 @@ where
     // Check for the optional "relay" subcommand.
     cfg_if::cfg_if! {
         if #[cfg(feature = "relay")] {
-            if let Some(_relay_matches) = matches.subcommand_matches("relay") {
-                // TODO: Actually implement the launch of a relay.
-                todo!()
+            if let Some(relay_matches) = matches.subcommand_matches("relay") {
+                return subcommands::relay::run(runtime, relay_matches, &config);
             }
         }
     }
