@@ -71,7 +71,15 @@ pub(crate) fn watch_for_config_changes<R: Runtime>(
 
     let rt = runtime.clone();
     let () = runtime.clone().spawn(async move {
-        let res: anyhow::Result<()> = run_watcher(rt, sources, modules, watch_file, sighup_stream).await;
+        let res: anyhow::Result<()> = run_watcher(
+            rt,
+            sources,
+            modules,
+            watch_file,
+            sighup_stream,
+            Some(DEBOUNCE_INTERVAL)
+        ).await;
+
         match res {
             Ok(()) => debug!("Config watcher task exiting"),
             // TODO: warn_report does not work on anyhow::Error.
@@ -91,6 +99,7 @@ async fn run_watcher<R: Runtime>(
     modules: Vec<Weak<dyn ReconfigurableModule>>,
     watch_file: bool,
     mut sighup_stream: impl Stream<Item = ()> + Unpin,
+    debounce_interval: Option<Duration>,
 ) -> anyhow::Result<()> {
     let (tx, mut rx) = file_watcher::channel();
     let mut watcher = if watch_file {
@@ -121,7 +130,9 @@ async fn run_watcher<R: Runtime>(
                 ).await?;
             },
             event = rx.next().fuse() => {
-                runtime.sleep(DEBOUNCE_INTERVAL).await;
+                if let Some(debounce_interval) = debounce_interval {
+                    runtime.sleep(debounce_interval).await;
+                }
 
                 while let Some(_ignore) = rx.try_recv() {
                     // Discard other events, so that we only reload once.
@@ -357,8 +368,11 @@ mod test {
 
     /// Write `data` to file `name` within `dir`.
     fn write_file(dir: &TestTempDir, name: &str, data: &[u8]) -> PathBuf {
+        let tmp = dir.as_path_untracked().join("tmp");
+        std::fs::write(&tmp, data).unwrap();
         let path = dir.as_path_untracked().join(name);
-        std::fs::write(&path, data).unwrap();
+        // Atomically write the config file
+        std::fs::rename(tmp, &path).unwrap();
         path
     }
 
@@ -369,7 +383,7 @@ mod test {
     }
 
     #[test]
-    #[ignore] // TODO(#1549): Re-enable
+    #[ignore] // TODO(#1607): Re-enable
     fn watch_single_file() {
         tor_rtcompat::test_with_one_runtime!(|rt| async move {
             let temp_dir = test_temp_dir!();
@@ -393,6 +407,7 @@ mod test {
                     vec![Arc::downgrade(&module)],
                     true,
                     sighup_rx,
+                    None,
                 ).await.unwrap();
             }).unwrap();
 
@@ -439,6 +454,7 @@ mod test {
                     vec![Arc::downgrade(&module)],
                     true,
                     sighup_rx,
+                    None,
                 ).await.unwrap();
             }).unwrap();
 
