@@ -21,8 +21,8 @@ use tor_config::Listen;
 use tor_error::warn_report;
 #[cfg(feature = "rpc")]
 use tor_rpcbase::{self as rpc};
-use tor_rtcompat::{Runtime, TcpListener};
-use tor_socksproto::{SocksAddr, SocksAuth, SocksCmd, SocksRequest};
+use tor_rtcompat::{NetStreamListener, Runtime};
+use tor_socksproto::{SocksAddr, SocksAuth, SocksCmd, SocksRequest, SOCKS_BUF_LEN};
 
 use anyhow::{anyhow, Context, Result};
 
@@ -39,11 +39,12 @@ Content-Type: text/html; charset=utf-8
 <title>This is a SOCKS Proxy, Not An HTTP Proxy</title>
 </head>
 <body>
-<h1>This is a SOCKs proxy, not an HTTP proxy.</h1>
+<h1>This is a SOCKS proxy, not an HTTP proxy.</h1>
 <p>
 It appears you have configured your web browser to use this Tor port as
 an HTTP proxy.
-</p><p>
+</p>
+<p>
 This is not correct: This port is configured as a SOCKS proxy, not
 an HTTP proxy. If you need an HTTP proxy tunnel, wait for Arti to
 add support for it in place of, or in addition to, socks_port.
@@ -87,10 +88,16 @@ struct SocksIsolationKey(ConnIsolation, ProvidedIsolation);
 #[derive(Debug, Clone, PartialEq, Eq)]
 enum ProvidedIsolation {
     /// The socks isolation itself.
-    Auth(SocksAuth),
-    /// A string provided as isolation with an RPC connection
-    #[cfg(feature = "rpc")]
-    RpcString(Option<String>),
+    Legacy(SocksAuth),
+    /// A bytestring provided as isolation with the extended Socks5 username/password protocol.
+    Extended {
+        /// Which format was negotiated?
+        ///
+        /// (At present, different format codes can't share a circuit.)
+        format_code: u8,
+        /// What's the isolation string?
+        isolation: Box<[u8]>,
+    },
 }
 
 impl arti_client::isolation::IsolationHelper for SocksIsolationKey {
@@ -109,62 +116,244 @@ impl arti_client::isolation::IsolationHelper for SocksIsolationKey {
 
 /// The meaning of a SOCKS authentication field, according to our conventions.
 struct AuthInterpretation {
-    /// Assign this stream to a client determined by given RPC session, and
-    /// register its existence with that session.
+    /// Associate this stream with a DataStream created by using a particular RPC object
+    /// as a Tor client.
     #[cfg(feature = "rpc")]
-    assign_to_session: Option<rpc::ObjectId>,
+    rpc_object: Option<rpc::ObjectId>,
 
     /// Isolate this stream from other streams that do not have the same
     /// value.
     isolation: ProvidedIsolation,
 }
 
+/// NOTE: The following documentation belongs in a spec.
+/// But for now, it's our best attempt to document the design and protocol
+/// implemented here
+/// for integrating SOCKS with our RPC system. --nickm
+///
+/// Roughly speaking:
+///
+/// ## Key concepts
+///
+/// A data stream is "RPC-visible" if, when it is created via SOCKS,
+/// the RPC system is told about it.
+///
+/// Every RPC-visible stream is associated with a given RPC object when it is created.
+/// (Since the RPC object is being specified in the SOCKS protocol,
+/// it must be one with an externally visible Object ID.
+/// Such Object IDs are cryptographically unguessable and unforgeable,
+/// and are qualified with a unique identifier for their associated RPC session.)
+/// Call this RPC Object the "target" object for now.
+/// This target RPC object must implement
+/// the [`ConnectWithPrefs`](arti_client::rpc::ConnectWithPrefs) special method.
+///
+/// Right now, there are two general kinds of objects that implement this method:
+/// client-like objects, and stream-like objects.
+///
+/// A client-like object is either a `TorClient` or an RPC `Session`.
+/// It knows about and it is capable of opening multiple data streams.
+/// Using it as the target object for a SOCKS connection tells Arti
+/// that the resulting data stream (if any)
+/// should be built by it, and associated with its RPC session.
+///
+/// An application gets a TorClient by asking the session for one,
+/// or for asking a TorClient to give you a new variant clone of itself.
+///
+/// A stream-like object is an `arti_rpcserver::stream::RpcDataStream`.
+/// It is created from a client-like object, but represents a single data stream.
+/// When created, it it not yet connected or trying to connect to anywhere:
+/// the act of using it as the target Object for a SOCKS connection causes
+/// it to begin connecting.
+/// (You can also think of this as a single-use client,
+/// which once used, becomes interchangeable with the DataStream it created.)
+/// (TODO: We may wish to change this vocabulary.
+/// We may wish to call this a "stream handle", for instance?)
+///
+/// An application gets an RpcDataStream by calling `arti:new_stream_handle
+/// on any client-like object.  Currently, this always creates an RpcDataStream
+/// that makes optimistic connections; See #1583.
+///
+/// ## The SOCKS protocol
+///
+/// See [proposal 351](https://spec.torproject.org/proposals/351-socks-auth-extensions.html) for now.
+/// Once it is merged, see the
+/// [SOCKS extensions spec](https://spec.torproject.org/socks-extensions.html).
+///
+/// ### Another proposed change
+///
+/// We could add a new method to clients, with a name like
+/// "open_stream" or "connect_stream".
+/// This method would include all target and isolation information in its parameters.
+/// It would actually create a DataStream immediately, tell it to begin connecting,
+/// and return an externally visible object ID.
+/// The RPC protocol could be used to watch the DataStream object,
+/// to see when it was connected.
+///
+/// The resulting DataStream object could also be used as the target of a SOCKS connection.
+/// We would require in such a case that no isolation be provided in the SOCKS handshake,
+/// and that the target address was (e.g.) INADDR_ANY.
+///
+/// ## Intended use cases (examples)
+///
+/// (These examples assume that the application
+/// already knows the SOCKS port it should use.
+/// I'm leaving out the isolation strings as orthogonal.)
+///
+/// These are **NOT** the only possible use cases;
+/// they're just the two that help understand this system best (I hope).
+///
+/// ### Case 1: Using a client-like object directly.
+///
+/// Here the application has authenticated to RPC
+/// and gotten the session ID `SESSION-1`.
+/// (In reality, this would be a longer ID, and full of crypto).
+///
+/// The application wants to open a new stream to www.example.com.
+/// They don't particularly care about isolation,
+/// but they do want their stream to use their RPC session.
+/// They don't want an Object ID for the stream.
+///
+/// To do this, they make a SOCKS connection to arti,
+/// with target address www.example.com.
+/// They set the username to `<torS0X>0SESSION-1`,
+/// and the password to the empty string.
+///
+/// Arti looks up the Session object via the `SESSION-1` object ID
+/// and tells it (via the ConnectWithPrefs special method)
+/// to connect to www.example.com.
+/// The session creates a new DataStream using its internal TorClient,
+/// but does not register the stream with an RPC Object ID.
+/// Arti proxies the application's SOCKS connection through this DataStream.
+///
+///
+/// ### Case 2: Creating an identifiable stream.
+///
+/// Here the application wants to be able to refer to its DataStream
+/// after the stream is created.
+/// As before, we assume that it's on an RPC session
+/// where the Session ID is `SESSION-1`.
+///
+/// The application sends an RPC request of the form:
+/// `{"id": 123, "obj": "SESSION-1", "method": "arti:new_stream_handle", "params": {}}`
+///
+/// It receives a reply like:
+/// `{"id": 123, "result": {"id": "STREAM-1"} }`
+///
+/// (In reality, `STREAM-1` would also be longer and full of crypto.)
+///
+/// Now the application has an object called `STREAM-1` that is not yet a connected
+/// stream, but which may become one.
+///
+/// This time, it wants to set its isolation string to "xyzzy".
+///
+/// The application opens a socks connection as before.
+/// For the username it sends `<torS0X>0STREAM-1`,
+/// and for the password it sends `xyzzy`.
+///
+/// Now Arti looks up the `RpcDataStream` object via `STREAM-1`,
+/// and tells it (via the ConnectWithPrefs special method)
+/// to connect to www.example.com.
+/// This causes the `RpcDataStream` internally to create a new `DataStream`,
+/// and to store that `DataStream` in itself.
+/// The `RpcDataStream` with Object ID `STREAM-1`
+/// is now an alias for the newly created `DataStream`.
+/// Arti proxies the application's SOCKS connection through that `DataStream`.
+///
+#[cfg(feature = "rpc")]
+#[allow(dead_code)]
+mod socks_and_rpc {}
+
 /// Given the authentication object from a socks connection, determine what it's telling
 /// us to do.
 ///
 /// (In no case is it actually SOCKS authentication: it can either be a message
 /// to the stream isolation system or the RPC system.)
 fn interpret_socks_auth(auth: &SocksAuth) -> Result<AuthInterpretation> {
-    // TODO RPC: This whole function and the way that it parses SOCKS
-    // authentication is a placeholder (because we need to put _something_ here
-    // for now).  We could probably come up with a much better design, and
-    // should.
-    //
-    // TODO RPC: In our final design we should probably figure out way to
-    // migrate away from the current "anything goes" approach to stream
-    // isolation without breaking all the existing apps that think they can use
-    // an arbitrary byte-string as their isolation token.
-
-    /// A constant which, when it appears as a username, indicates that the
-    /// stream is to be assigned to an Arti RPC session.
-    const RPC_SESSION_CONST: &[u8] = b"<arti-rpc-session>";
-
-    #[allow(unused_variables)] // TODO RPC remove
-    match auth {
-        SocksAuth::Username(user, pass) if user == RPC_SESSION_CONST => {
-            cfg_if::cfg_if! {
-                if #[cfg(feature="rpc")] {
-                    let pass =
-                        std::str::from_utf8(pass).context("rpc-session info must be utf-8")?;
-                    let (session, isolation) = match pass.split_once(':') {
-                        Some((s,i)) => (s.to_owned().into(), Some(i.to_owned())),
-                        None => (pass.to_owned().into(), None),
-                    };
-                    Ok(AuthInterpretation {
-                        assign_to_session: Some(session),
-                        isolation: ProvidedIsolation::RpcString(isolation)
-                    })
-                } else {
-                    Err(anyhow!("Not built with support for RPC"))
-                }
-            }
-        }
-        other_auth => Ok(AuthInterpretation {
-            #[cfg(feature = "rpc")]
-            assign_to_session: None,
-            isolation: ProvidedIsolation::Auth(other_auth.clone()),
-        }),
+    /// Interpretation of a SOCKS5 username according to Prop351.
+    enum Uname<'a> {
+        /// This is a legacy username; it's just part of the
+        /// isolation information.
+        //
+        // Note: We're not actually throwing away the username here;
+        // instead we're going to use the whole SocksAuth
+        // in a `ProvidedAuthentication::Legacy``.
+        // TODO RPC: Find a more idiomatic way to express this data flow.
+        Legacy,
+        /// This is using the prop351 socks extension: contains the extension
+        /// format code and the remaining information from the username.
+        Extended(u8, &'a [u8]),
     }
+    /// Helper: Try to interpret a SOCKS5 username field as indicating the start of a set of
+    /// extended socks authentication information.
+    ///
+    /// Implements Prop351.
+    ///
+    /// If it does indicate that extensions are in use,
+    /// return a `Uname::Extended` containing
+    /// the extension format type and the remaining information from the username.
+    ///
+    /// If it indicates that no extensions are in use,
+    /// return `Uname::Legacy`.
+    ///
+    /// If it is badly formatted, return an error.
+    fn interpret_socks5_username(username: &[u8]) -> Result<Uname<'_>> {
+        /// 8-byte "magic" sequence from Prop351.
+        /// When it appears at the start of a username,
+        /// indicates that the username/password are to be interpreted as
+        /// as encoding SOCKS5 extended parameters,
+        /// but the format might not be one we recognize.
+        const SOCKS_EXT_CONST_ANY: &[u8] = b"<torS0X>";
+        let Some(remainder) = username.strip_prefix(SOCKS_EXT_CONST_ANY) else {
+            return Ok(Uname::Legacy);
+        };
+        if remainder.is_empty() {
+            return Err(anyhow!("Exteneded SOCKS information without format code."));
+        }
+        // TODO MSRV 1.80: use split_at_checked instead.
+        // This won't panic since we checked for an empty string above.
+        let (format_code, remainder) = remainder.split_at(1);
+        Ok(Uname::Extended(format_code[0], remainder))
+    }
+
+    let isolation = match auth {
+        SocksAuth::Username(user, pass) => match interpret_socks5_username(user)? {
+            Uname::Legacy => ProvidedIsolation::Legacy(auth.clone()),
+            Uname::Extended(b'1', b"") => {
+                return Err(anyhow!("Received empty RPC object ID"));
+            }
+            Uname::Extended(format_code @ b'1', remainder) => {
+                #[cfg(not(feature = "rpc"))]
+                return Err(anyhow!(
+                    "Received RPC object ID, but not built with support for RPC"
+                ));
+                #[cfg(feature = "rpc")]
+                return Ok(AuthInterpretation {
+                    rpc_object: Some(rpc::ObjectId::from(
+                        std::str::from_utf8(remainder).context("Rpc object ID was not utf-8")?,
+                    )),
+                    isolation: ProvidedIsolation::Extended {
+                        format_code,
+                        isolation: pass.clone().into(),
+                    },
+                });
+            }
+            Uname::Extended(format_code @ b'0', b"") => ProvidedIsolation::Extended {
+                format_code,
+                isolation: pass.clone().into(),
+            },
+            Uname::Extended(b'0', _) => {
+                return Err(anyhow!("Extraneous information in SOCKS username field."))
+            }
+            _ => return Err(anyhow!("Unrecognized SOCKS format code")),
+        },
+        _ => ProvidedIsolation::Legacy(auth.clone()),
+    };
+
+    Ok(AuthInterpretation {
+        #[cfg(feature = "rpc")]
+        rpc_object: None,
+        isolation,
+    })
 }
 
 /// Information used to implement a SOCKS connection.
@@ -216,7 +405,7 @@ impl<R: Runtime> SocksConnContext<R> {
         prefs.set_isolation(SocksIsolationKey(conn_isolation, interp.isolation));
 
         #[cfg(feature = "rpc")]
-        if let Some(session) = interp.assign_to_session {
+        if let Some(session) = interp.rpc_object {
             if let Some(mgr) = &self.rpc_mgr {
                 let (context, object) = mgr
                     .lookup_object(&session)
@@ -276,10 +465,15 @@ where
             return Err(anyhow!("Socks handshake did not fit in 1KiB buffer"));
         }
         // Read some more stuff.
-        n_read += socks_r
+        let n = socks_r
             .read(&mut inbuf[n_read..])
             .await
             .context("Error while reading SOCKS handshake")?;
+        if n == 0 {
+            debug!("Socks connection closed");
+            return Ok(());
+        }
+        n_read += n;
 
         // try to advance the handshake to the next state.
         let action = match handshake.handshake(&inbuf[..n_read]) {
@@ -303,7 +497,7 @@ where
 
         // reply if needed.
         if action.drain > 0 {
-            inbuf.copy_within(action.drain..action.drain + n_read, 0);
+            inbuf.copy_within(action.drain..n_read, 0);
             n_read -= action.drain;
         }
         if !action.reply.is_empty() {
@@ -320,6 +514,11 @@ where
             return Ok(());
         }
     };
+    if n_read != 0 {
+        // TODO this is not great, we should actually *support* optimistic data.
+        // But this is an easy way to mitigate #1627 aka TROVE-2024-010 for now.
+        return Err(anyhow!("Optimistic data received - sent by client before SOCKS handshake completed.  This is not supported in this version of Arti, sorry."));
+    }
 
     // Unpack the socks request and find out where we're connecting to.
     let addr = request.addr().to_string();
@@ -538,7 +737,7 @@ where
 {
     use futures::{poll, task::Poll};
 
-    let mut buf = [0_u8; 1024];
+    let mut buf = [0_u8; SOCKS_BUF_LEN];
 
     // At this point we could just loop, calling read().await,
     // write_all().await, and flush().await.  But we want to be more
@@ -673,7 +872,7 @@ pub(crate) async fn run_socks_proxy<R: Runtime>(
     let mut incoming = futures::stream::select_all(
         listeners
             .into_iter()
-            .map(TcpListener::incoming)
+            .map(NetStreamListener::incoming)
             .enumerate()
             .map(|(listener_id, incoming_conns)| {
                 incoming_conns.map(move |socket| (socket, listener_id))
