@@ -3,9 +3,10 @@
 #[cfg(feature = "onion-service-service")]
 pub(crate) mod hss;
 
-#[cfg(all(
-    feature = "onion-service-client",
-    feature = "experimental-api",
-    feature = "keymgr"
-))]
+#[cfg(feature = "hsc")]
 pub(crate) mod hsc;
+
+#[cfg(feature = "relay")]
+pub(crate) mod relay;
+
+pub(crate) mod proxy;
