@@ -46,7 +46,8 @@ mod prepare_send;
 mod sinkext;
 mod watch;
 
-pub mod oneshot;
+pub mod peekable_stream;
+pub mod stream_peek;
 
 pub use join_read_write::*;
 
@@ -55,3 +56,5 @@ pub use prepare_send::{SinkPrepareExt, SinkPrepareSendFuture, SinkSendable};
 pub use sinkext::SinkExt;
 
 pub use watch::{DropNotifyEofSignallable, DropNotifyWatchSender, PostageWatchSenderExt};
+
+pub use oneshot_fused_workaround as oneshot;
