@@ -3,7 +3,7 @@
 use std::io;
 use std::sync::{Arc, Mutex};
 
-use crate::factory::{BootstrapReporter, ChannelFactory};
+use crate::factory::{BootstrapReporter, ChannelFactory, IncomingChannelFactory};
 use crate::transport::TransportImplHelper;
 use crate::{event::ChanMgrEventSender, Error};
 
@@ -84,6 +84,37 @@ where
     }
 }
 
+#[async_trait]
+impl<R: Runtime, H: TransportImplHelper> IncomingChannelFactory for ChanBuilder<R, H>
+where
+    R: tor_rtcompat::TlsProvider<H::Stream> + Send + Sync,
+    H: Send + Sync,
+{
+    type Stream = H::Stream;
+
+    #[cfg(feature = "relay")]
+    async fn accept_from_transport(
+        &self,
+        peer: std::net::SocketAddr,
+        stream: Self::Stream,
+    ) -> crate::Result<Arc<tor_proto::channel::Channel>> {
+        let map_ioe = |ioe, action| Error::Io {
+            action,
+            peer: Some(BridgeAddr::new_addr_from_sockaddr(peer).into()),
+            source: ioe,
+        };
+
+        let _tls = self
+            .tls_connector
+            .negotiate_unvalidated(stream, "ignored")
+            .await
+            .map_err(|e| map_ioe(e.into(), "TLS negotiation"))?;
+
+        // TODO RELAY: do handshake and build channel
+        todo!();
+    }
+}
+
 impl<R: Runtime, H: TransportImplHelper> ChanBuilder<R, H>
 where
     R: tor_rtcompat::TlsProvider<H::Stream> + Send + Sync,
@@ -242,13 +273,14 @@ mod test {
         mgr::{AbstractChannel, AbstractChannelFactory},
         Result,
     };
+    use futures::StreamExt as _;
     use std::net::SocketAddr;
     use std::time::{Duration, SystemTime};
     use tor_linkspec::{ChannelMethod, HasRelayIds, RelayIdType};
     use tor_llcrypto::pk::ed25519::Ed25519Identity;
     use tor_llcrypto::pk::rsa::RsaIdentity;
     use tor_proto::channel::Channel;
-    use tor_rtcompat::{test_with_one_runtime, TcpListener};
+    use tor_rtcompat::{test_with_one_runtime, NetStreamListener};
     use tor_rtmock::{io::LocalStream, net::MockNetwork, MockSleepRuntime};
 
     // Make sure that the builder can build a real channel.  To test
@@ -310,7 +342,12 @@ mod test {
                 async {
                     // relay-side: accept the channel
                     // (and pretend to know what we're doing).
-                    let (mut con, addr) = lis.accept().await.expect("accept failed");
+                    let (mut con, addr) = lis
+                        .incoming()
+                        .next()
+                        .await
+                        .expect("Closed?")
+                        .expect("accept failed");
                     assert_eq!(client_addr, addr.ip());
                     crate::testing::answer_channel_req(&mut con)
                         .await
