@@ -567,6 +567,8 @@ mod test {
 
         type BuildSpec = tor_linkspec::OwnedChanTarget;
 
+        type Stream = ();
+
         async fn build_channel(
             &self,
             _target: &Self::BuildSpec,
@@ -574,6 +576,15 @@ mod test {
         ) -> Result<Arc<FakeChannel>> {
             unimplemented!()
         }
+
+        #[cfg(feature = "relay")]
+        async fn build_channel_using_incoming(
+            &self,
+            _peer: std::net::SocketAddr,
+            _stream: Self::Stream,
+        ) -> Result<Arc<Self::Channel>> {
+            unimplemented!()
+        }
     }
 
     #[derive(Clone, Debug)]
