@@ -22,9 +22,10 @@ use std::{
 use futures::{AsyncReadExt, AsyncWriteExt};
 use tor_error::internal;
 use tor_linkspec::PtTargetAddr;
-use tor_rtcompat::TcpProvider;
+use tor_rtcompat::NetStreamProvider;
 use tor_socksproto::{
     SocksAddr, SocksAuth, SocksClientHandshake, SocksCmd, SocksRequest, SocksStatus, SocksVersion,
+    SOCKS_BUF_LEN,
 };
 use tracing::trace;
 
@@ -64,12 +65,12 @@ const NO_ADDR: IpAddr = IpAddr::V4(std::net::Ipv4Addr::new(0, 0, 0, 1));
 //
 // TODO: Perhaps we should refactor this someday so it can be a general-purpose
 // proxy function, not only for Arti.
-pub(crate) async fn connect_via_proxy<R: TcpProvider + Send + Sync>(
+pub(crate) async fn connect_via_proxy<R: NetStreamProvider + Send + Sync>(
     runtime: &R,
     proxy: &SocketAddr,
     protocol: &Protocol,
     target: &PtTargetAddr,
-) -> Result<R::TcpStream, ProxyError> {
+) -> Result<R::Stream, ProxyError> {
     trace!(
         "Launching a proxied connection to {} via proxy at {} using {:?}",
         target,
@@ -112,7 +113,7 @@ pub(crate) async fn connect_via_proxy<R: TcpProvider + Send + Sync>(
     // TODO: This code is largely copied from the socks server wrapper code in
     // arti::proxy. Perhaps we should condense them into a single thing, if we
     // don't just revise the SOCKS code completely.
-    let mut inbuf = [0_u8; 1024];
+    let mut inbuf = [0_u8; SOCKS_BUF_LEN];
     let mut n_read = 0;
     let reply = loop {
         // try to advance the handshake to the next state.
@@ -136,7 +137,7 @@ pub(crate) async fn connect_via_proxy<R: TcpProvider + Send + Sync>(
 
         // reply if needed.
         if action.drain > 0 {
-            inbuf.copy_within(action.drain..action.drain + n_read, 0);
+            inbuf.copy_within(action.drain..n_read, 0);
             n_read -= action.drain;
         }
         if !action.reply.is_empty() {
@@ -163,7 +164,11 @@ pub(crate) async fn connect_via_proxy<R: TcpProvider + Send + Sync>(
         }
 
         // Read some more stuff.
-        n_read += stream.read(&mut inbuf[n_read..]).await?;
+        let n = stream.read(&mut inbuf[n_read..]).await?;
+        if n == 0 {
+            return Err(ProxyError::SocksClosed);
+        }
+        n_read += n;
     };
 
     let status = reply
@@ -214,6 +219,10 @@ pub enum ProxyError {
     #[error("Protocol error while communicating with SOCKS proxy")]
     SocksProto(#[source] tor_socksproto::Error),
 
+    /// Unexpected close from SOCKS proxy before handshake was completed.
+    #[error("Unexpected close while communicating with SOCKS proxy")]
+    SocksClosed,
+
     /// We encountered an internal programming error.
     #[error("Internal error")]
     Bug(#[from] tor_error::Bug),
@@ -251,6 +260,7 @@ impl tor_error::HasKind for ProxyError {
             E::InvalidSocksAddr(_) | E::InvalidSocksRequest(_) => EK::BadApiUsage,
             E::UnrecognizedAddr => EK::NotImplemented,
             E::SocksProto(_) => EK::LocalProtocolViolation,
+            E::SocksClosed => EK::LocalNetworkError, // Could also be a protocol violation.
             E::Bug(e) => e.kind(),
             E::UnexpectedData => EK::NotImplemented,
             E::SocksError(_) => EK::LocalProtocolViolation,
@@ -269,6 +279,7 @@ impl tor_error::HasRetryTime for ProxyError {
             E::UnrecognizedAddr => RT::Never,
             E::InvalidSocksRequest(_) => RT::Never,
             E::SocksProto(_) => RT::AfterWaiting,
+            E::SocksClosed => RT::AfterWaiting,
             E::Bug(_) => RT::Never,
             E::UnexpectedData => RT::Never,
             E::SocksError(e) => match *e {
@@ -299,7 +310,7 @@ pub struct ExternalProxyPlugin<R> {
 
 #[cfg(feature = "pt-client")]
 #[cfg_attr(docsrs, doc(cfg(feature = "pt-client")))]
-impl<R: TcpProvider + Send + Sync> ExternalProxyPlugin<R> {
+impl<R: NetStreamProvider + Send + Sync> ExternalProxyPlugin<R> {
     /// Make a new `ExternalProxyPlugin`.
     pub fn new(rt: R, proxy_addr: SocketAddr, proxy_version: SocksVersion) -> Self {
         Self {
@@ -312,13 +323,13 @@ impl<R: TcpProvider + Send + Sync> ExternalProxyPlugin<R> {
 
 #[cfg(feature = "pt-client")]
 #[async_trait]
-impl<R: TcpProvider + Send + Sync> TransportImplHelper for ExternalProxyPlugin<R> {
-    type Stream = R::TcpStream;
+impl<R: NetStreamProvider + Send + Sync> TransportImplHelper for ExternalProxyPlugin<R> {
+    type Stream = R::Stream;
 
     async fn connect(
         &self,
         target: &OwnedChanTarget,
-    ) -> crate::Result<(OwnedChanTarget, R::TcpStream)> {
+    ) -> crate::Result<(OwnedChanTarget, R::Stream)> {
         let pt_target = match target.chan_method() {
             ChannelMethod::Direct(_) => {
                 return Err(crate::Error::UnusableTarget(bad_api_usage!(
