@@ -113,6 +113,12 @@ impl From<&HsIdKeypair> for HsIdKey {
     }
 }
 
+impl From<HsIdKeypair> for HsIdKey {
+    fn from(value: HsIdKeypair) -> Self {
+        Self(*value.0.public())
+    }
+}
+
 /// VERSION from rend-spec-v3 s.6 \[ONIONADDRESS]
 const HSID_ONION_VERSION: u8 = 0x03;
 
@@ -385,6 +391,12 @@ define_pk_keypair! {
 pub struct HsBlindIdKey(ed25519::PublicKey) / HsBlindIdKeypair(ed25519::ExpandedKeypair);
 }
 
+impl From<HsBlindIdKeypair> for HsBlindIdKey {
+    fn from(kp: HsBlindIdKeypair) -> HsBlindIdKey {
+        HsBlindIdKey(kp.0.into())
+    }
+}
+
 define_bytes! {
 /// A blinded onion service identity, represented in a compact format. (`KP_hs_blind_id`)
 ///
