@@ -33,7 +33,6 @@ pub(crate) use {
     std::io::{BufReader, BufWriter, Read, Seek, SeekFrom, Write},
     std::iter,
     std::marker::PhantomData,
-    std::ops::{Deref, DerefMut},
     std::panic::AssertUnwindSafe,
     std::path::{Path, PathBuf},
     std::str::FromStr,
@@ -86,7 +85,7 @@ pub(crate) use {
     tor_error::{Bug, ErrorKind, ErrorReport as _, HasKind},
     tor_hscrypto::ope::AesOpeKey,
     tor_hscrypto::pk::{
-        HsBlindId, HsBlindIdKey, HsBlindIdKeypair, HsClientDescEncKey, HsDescSigningKeypair, HsId,
+        HsBlindId, HsBlindIdKey, HsBlindIdKeypair, HsClientDescEncKey, HsDescSigningKeypair,
         HsIdKey, HsIdKeypair, HsIntroPtSessionIdKey, HsIntroPtSessionIdKeypair, HsSvcNtorKeypair,
     },
     tor_hscrypto::time::TimePeriod,
@@ -149,7 +148,10 @@ pub(crate) use {
         ClientError,
     },
     crate::{req::RendRequestContext, HsNickname, LinkSpecs, NtorPublicKey},
-    crate::{BlindIdKeypairSpecifier, DescSigningKeypairSpecifier, HsIdKeypairSpecifier},
+    crate::{
+        BlindIdKeypairSpecifier, DescSigningKeypairSpecifier, HsIdKeypairSpecifier,
+        HsIdPublicKeySpecifier,
+    },
     crate::{DescUploadError, IptError},
     crate::{FatalError, RendRequest},
     ipt_establish::{IptEstablisher, IptParameters, IptStatus, IptStatusStatus, IptWantsToRetire},
