@@ -98,6 +98,7 @@ pub use keys::{
 pub use nickname::{HsNickname, InvalidNickname};
 pub use publish::UploadError as DescUploadError;
 pub use req::{RendRequest, StreamRequest};
+pub use tor_hscrypto::pk::HsId;
 
 pub use helpers::handle_rend_requests;
 
@@ -205,13 +206,12 @@ impl From<oneshot::Canceled> for ShutdownStatus {
 
 /// A handle to an instance of an onion service, which may or may not be running.
 ///
-/// To construct an `OnionService`, call [`OnionService::new`].
+/// To construct an `OnionService`, use [`OnionServiceBuilder`].
 /// It will not start handling requests until you call its
 /// [``.launch()``](OnionService::launch) method.
-//
-// TODO (#1228): Write more.
-// TODO (#1247): Choose a better name for this struct
-//
+///
+/// Note: the identity key (HsId) of the service is not generated until
+/// [``.launch()``](OnionService::launch) is called.
 #[derive(Builder)]
 #[builder(build_fn(private, name = "build_unvalidated", error = "FatalError"))]
 pub struct OnionService {
@@ -224,41 +224,6 @@ pub struct OnionService {
 }
 
 impl OnionService {
-    /// Create (but do not launch) a new onion service.
-    ///
-    /// The onion service's behavior and local nickname will be determined by
-    /// the provided configuration.
-    ///
-    /// The onion service's keys will be kept in `keymgr`.
-    /// Its non-key state information will be stored persistently in a location
-    /// and manner controlled by `state_dir`.
-    ///
-    // TODO (#1228): document.
-    //
-    // TODO (#1228): Document how we handle the case where somebody tries to launch two
-    // onion services with the same nickname?  They will conflict by trying to
-    // use the same state and the same keys.  Do we stop it here, or in
-    // arti_client?
-    #[deprecated(since = "1.2.6", note = "Use OnionServiceBuilder instead.")]
-    pub fn new(
-        config: OnionServiceConfig,
-        keymgr: Arc<KeyMgr>,
-        state_dir: &StateDirectory,
-    ) -> Result<Self, StartupError> {
-        // TODO (#1194): add a config option for specifying whether to expect the KS_hsid to be stored
-        // offline
-        //let offline_hsid = config.offline_hsid;
-        let offline_hsid = false;
-
-        maybe_generate_hsid(&keymgr, &config.nickname, offline_hsid)?;
-
-        Ok(OnionService {
-            config,
-            keymgr,
-            state_dir: state_dir.clone(),
-        })
-    }
-
     /// Create an [`OnionServiceBuilder`].
     pub fn builder() -> OnionServiceBuilder {
         OnionServiceBuilder::default()
@@ -290,6 +255,15 @@ impl OnionService {
 
         let nickname = config.nickname.clone();
 
+        // TODO (#1194): add a config option for specifying whether to expect the KS_hsid to be stored
+        // offline
+        //let offline_hsid = config.offline_hsid;
+        let offline_hsid = false;
+
+        // TODO (#1106): make this configurable
+        let selector = KeystoreSelector::Primary;
+        maybe_generate_hsid(&keymgr, &config.nickname, offline_hsid, selector)?;
+
         if config.restricted_discovery.enabled {
             info!(
                 nickname=%nickname,
@@ -379,20 +353,39 @@ impl OnionService {
     pub fn onion_name(&self) -> Option<HsId> {
         onion_name(&self.keymgr, &self.config.nickname)
     }
+
+    /// Generate an identity key (KP_hs_id) for this service.
+    ///
+    /// If the keystore specified by `selector` contains an entry for the identity key
+    /// of this service, it will be returned. Otherwise, a new key will be generated.
+    ///
+    /// Most users do not need to call this function: on [`launch`](`OnionService::launch`),
+    /// the service will automatically generate its identity key if needed.
+    /// You should only use this function if you need to know the KP_hs_id of the service
+    /// before launching it.
+    ///
+    /// The `selector` argument is used for choosing the keystore in which to generate the keypair.
+    /// While most users will want to write to the [`Primary`](KeystoreSelector::Primary), if you
+    /// have configured this `TorClient` with a non-default keystore and wish to generate the
+    /// keypair in it, you can do so by calling this function with a [KeystoreSelector::Id]
+    /// specifying the keystore ID of your keystore.
+    ///
+    // Note: the selector argument exists for future-proofing reasons. We don't currently support
+    // configuring custom or non-default keystores (see #1106).
+    pub fn generate_identity_key(&self, selector: KeystoreSelector) -> Result<HsId, StartupError> {
+        // TODO (#1194): add a config option for specifying whether to expect the KS_hsid to be stored
+        // offline
+        //let offline_hsid = config.offline_hsid;
+        let offline_hsid = false;
+
+        maybe_generate_hsid(&self.keymgr, &self.config.nickname, offline_hsid, selector)
+    }
 }
 
 impl OnionServiceBuilder {
     /// Build the [`OnionService`]
     pub fn build(&self) -> Result<OnionService, StartupError> {
         let svc = self.build_unvalidated()?;
-
-        // TODO (#1194): add a config option for specifying whether to expect the KS_hsid to be stored
-        // offline
-        //let offline_hsid = config.offline_hsid;
-        let offline_hsid = false;
-
-        maybe_generate_hsid(&svc.keymgr, &svc.config.nickname, offline_hsid)?;
-
         Ok(svc)
     }
 }
@@ -419,8 +412,6 @@ impl RunningOnionService {
             })
         })
 
-        // TODO: reconfigure the authorized_clients too
-
         // TODO (#1153, #1209): We need to make sure that the various tasks listening on
         // config_rx actually enforce the configuration, not only on new
         // connections, but existing ones.
@@ -508,29 +499,29 @@ fn maybe_generate_hsid(
     keymgr: &Arc<KeyMgr>,
     nickname: &HsNickname,
     offline_hsid: bool,
-) -> Result<(), StartupError> {
+    selector: KeystoreSelector,
+) -> Result<HsId, StartupError> {
     if offline_hsid {
         unimplemented!("offline hsid mode");
     }
 
-    let hsid_spec = HsIdKeypairSpecifier::new(nickname.clone());
+    let hsid_spec = HsIdPublicKeySpecifier::new(nickname.clone());
 
     let kp = keymgr
-        .get::<HsIdKeypair>(&hsid_spec)
+        .get::<HsIdKey>(&hsid_spec)
         .map_err(|cause| StartupError::Keystore {
             action: "read",
             cause,
         })?;
 
-    // TODO (#1106): make this configurable
-    let selector = KeystoreSelector::Default;
     let mut rng = rand::thread_rng();
-    let (keypair, generated) = match kp {
-        Some(kp) => (kp, false),
+    let (hsid, generated) = match kp {
+        Some(kp) => (kp.id(), false),
         None => {
             // Note: there is a race here. If the HsId is generated through some other means
             // (e.g. via the CLI) at some point between the time we looked up the keypair and
             // now, we will return an error.
+            let hsid_spec = HsIdKeypairSpecifier::new(nickname.clone());
             let kp = keymgr
                 .generate::<HsIdKeypair>(&hsid_spec, selector, &mut rng, false /* overwrite */)
                 .map_err(|cause| StartupError::Keystore {
@@ -538,11 +529,10 @@ fn maybe_generate_hsid(
                     cause,
                 })?;
 
-            (kp, true)
+            (HsIdKey::from(&kp).id(), true)
         }
     };
 
-    let hsid: HsId = HsIdKey::from(&keypair).id();
     if generated {
         info!(
             "Generated a new identity for service {nickname}: {}",
@@ -557,7 +547,7 @@ fn maybe_generate_hsid(
         );
     }
 
-    Ok(())
+    Ok(hsid)
 }
 
 /// Return the onion address of this service.
@@ -572,14 +562,12 @@ fn maybe_generate_hsid(
 // method on an ArtiHss type, and make both OnionService and RunningOnionService deref to
 // ArtiHss.
 fn onion_name(keymgr: &KeyMgr, nickname: &HsNickname) -> Option<HsId> {
-    let hsid_spec = HsIdKeypairSpecifier::new(nickname.clone());
+    let hsid_spec = HsIdPublicKeySpecifier::new(nickname.clone());
 
-    // TODO (#1194): This will need to be revisited when we implement offline hsid mode,
-    // (the HsId keypair won't be in the keystore)
     keymgr
-        .get::<HsIdKeypair>(&hsid_spec)
+        .get::<HsIdKey>(&hsid_spec)
         .ok()?
-        .map(|hsid| HsIdKey::from(&hsid).id())
+        .map(|hsid| hsid.id())
 }
 
 #[cfg(test)]
@@ -628,7 +616,7 @@ pub(crate) mod test {
 
             Arc::new(
                 KeyMgrBuilder::default()
-                    .default_store(Box::new(keystore))
+                    .primary_store(Box::new(keystore))
                     .build()
                     .unwrap(),
             )
@@ -675,7 +663,7 @@ pub(crate) mod test {
             assert!($keymgr.get::<HsIdKey>(&pub_hsid_spec).unwrap().is_none());
             assert!($keymgr.get::<HsIdKeypair>(&hsid_spec).unwrap().is_none());
 
-            maybe_generate_hsid(&$keymgr, &nickname, $offline_hsid).unwrap();
+            maybe_generate_hsid(&$keymgr, &nickname, $offline_hsid, Default::default()).unwrap();
         }};
     }
 
@@ -716,10 +704,21 @@ pub(crate) mod test {
         let existing_hsid_keypair = HsIdKeypair::from(existing_keypair);
 
         keymgr
-            .insert(existing_hsid_keypair, &hsid_spec, KeystoreSelector::Default)
+            .insert(
+                existing_hsid_keypair,
+                &hsid_spec,
+                KeystoreSelector::Primary,
+                true,
+            )
             .unwrap();
 
-        maybe_generate_hsid(&keymgr, &nickname, false /* offline_hsid */).unwrap();
+        maybe_generate_hsid(
+            &keymgr,
+            &nickname,
+            false, /* offline_hsid */
+            Default::default(),
+        )
+        .unwrap();
 
         let keypair = keymgr.get::<HsIdKeypair>(&hsid_spec).unwrap().unwrap();
         let pk: HsIdKey = (&keypair).into();
@@ -757,15 +756,21 @@ pub(crate) mod test {
         let (_hsid_keypair, hsid_public) = create_hsid();
 
         keymgr
-            .insert(hsid_keypair, &hsid_spec, KeystoreSelector::Default)
+            .insert(hsid_keypair, &hsid_spec, KeystoreSelector::Primary, true)
             .unwrap();
 
         // Insert a mismatched public key
         keymgr
-            .insert(hsid_public, &pub_hsid_spec, KeystoreSelector::Default)
+            .insert(hsid_public, &pub_hsid_spec, KeystoreSelector::Primary, true)
             .unwrap();
 
-        assert!(maybe_generate_hsid(&keymgr, &nickname, false /* offline_hsid */).is_err());
+        assert!(maybe_generate_hsid(
+            &keymgr,
+            &nickname,
+            false, /* offline_hsid */
+            Default::default()
+        )
+        .is_err());
     }
 
     #[test]
@@ -779,7 +784,7 @@ pub(crate) mod test {
 
         // Insert the hsid into the keystore
         keymgr
-            .insert(hsid_keypair, &hsid_spec, KeystoreSelector::Default)
+            .insert(hsid_keypair, &hsid_spec, KeystoreSelector::Primary, true)
             .unwrap();
 
         let config = OnionServiceConfigBuilder::default()
