@@ -6,7 +6,7 @@ use std::ops::Range;
 use std::result::Result as StdResult;
 use std::str::FromStr;
 
-use derive_more::{Deref, DerefMut, Display, From, Into};
+use derive_more::{Deref, DerefMut, From, Into};
 use thiserror::Error;
 use tor_error::{internal, into_internal, Bug};
 use tor_hscrypto::pk::{HsId, HsIdParseError, HSID_ONION_SUFFIX};
@@ -20,7 +20,7 @@ use crate::{ArtiPath, ArtiPathSyntaxError};
 pub mod derive;
 
 /// The identifier of a key.
-#[derive(Clone, Debug, PartialOrd, Ord, PartialEq, Eq, Hash, From, Display)]
+#[derive(Clone, Debug, PartialOrd, Ord, PartialEq, Eq, Hash, From, derive_more::Display)]
 #[non_exhaustive]
 pub enum KeyPath {
     /// An Arti key path.
@@ -301,7 +301,9 @@ pub enum KeyPathPattern {
 }
 
 /// The path of a key in the C Tor key store.
-#[derive(Clone, Debug, PartialOrd, Ord, PartialEq, Eq, Hash, Deref, DerefMut, Into, Display)]
+#[derive(
+    Clone, Debug, PartialOrd, Ord, PartialEq, Eq, Hash, Deref, DerefMut, Into, derive_more::Display,
+)]
 pub struct CTorPath(String);
 
 /// The "specifier" of a key, which identifies an instance of a key.
@@ -318,6 +320,10 @@ pub trait KeySpecifier {
     /// This function should return `None` for keys that are recognized by Arti's key stores, but
     /// not by C Tor's key store (such as `HsClientIntroAuthKeypair`).
     fn ctor_path(&self) -> Option<CTorPath>;
+
+    /// If this is the specifier for a public key, the specifier for
+    /// the corresponding (secret) keypair from which it can be derived
+    fn keypair_specifier(&self) -> Option<Box<dyn KeySpecifier>>;
 }
 
 /// A trait for serializing and deserializing specific types of [`Slug`]s.
@@ -372,6 +378,10 @@ impl KeySpecifier for ArtiPath {
     fn ctor_path(&self) -> Option<CTorPath> {
         None
     }
+
+    fn keypair_specifier(&self) -> Option<Box<dyn KeySpecifier>> {
+        None
+    }
 }
 
 impl KeySpecifier for CTorPath {
@@ -382,6 +392,10 @@ impl KeySpecifier for CTorPath {
     fn ctor_path(&self) -> Option<CTorPath> {
         Some(self.clone())
     }
+
+    fn keypair_specifier(&self) -> Option<Box<dyn KeySpecifier>> {
+        None
+    }
 }
 
 impl KeySpecifier for KeyPath {
@@ -398,6 +412,10 @@ impl KeySpecifier for KeyPath {
             KeyPath::CTor(p) => p.ctor_path(),
         }
     }
+
+    fn keypair_specifier(&self) -> Option<Box<dyn KeySpecifier>> {
+        None
+    }
 }
 
 impl KeySpecifierComponent for TimePeriod {
