@@ -394,6 +394,7 @@ define_derive_deftly! {
 
     #[doc = concat!("Pattern matching some or all [`", stringify!($tname), "`]")]
     #[allow(dead_code)] // Not everyone will need the pattern feature
+    #[non_exhaustive]
     $tvis struct $<$tname Pattern><$tdefgens>
     where $twheres
     ${vdefbody $vname $(
@@ -499,6 +500,14 @@ define_derive_deftly! {
                 None
             }}
         }
+
+        fn keypair_specifier(&self) -> Option<Box<dyn KeySpecifier>> {
+            ${if tmeta(keypair_specifier) {
+                Some(Box::new(std::convert::Into::<${tmeta(keypair_specifier) as token_stream}>::into(self)))
+            } else {
+                None
+            }}
+        }
     }
 
     impl<$tgens> $crate::KeySpecifierPattern for $<$tname Pattern><$tdefgens>
