@@ -3,24 +3,24 @@
 //! See the [`KeyMgr`] docs for more details.
 
 use crate::{
-    BoxedKeystore, EncodableKey, KeyPath, KeyPathError, KeyPathInfo, KeyPathInfoExtractor,
-    KeyPathPattern, KeySpecifier, KeyType, Keygen, KeygenRng, KeystoreId, KeystoreSelector, Result,
-    ToEncodableKey,
+    BoxedKeystore, KeyPath, KeyPathError, KeyPathInfo, KeyPathInfoExtractor, KeyPathPattern,
+    KeySpecifier, KeystoreId, KeystoreSelector, Result,
 };
 
 use itertools::Itertools;
 use std::iter;
 use std::result::Result as StdResult;
 use tor_error::{bad_api_usage, internal};
+use tor_key_forge::{EncodableKey, KeyType, Keygen, KeygenRng, ToEncodableKey};
 
-/// A key manager that acts as a frontend to a default [`Keystore`](crate::Keystore) and
+/// A key manager that acts as a frontend to a primary [`Keystore`](crate::Keystore) and
 /// any number of secondary [`Keystore`](crate::Keystore)s.
 ///
 /// Note: [`KeyMgr`] is a low-level utility and does not implement caching (the key stores are
 /// accessed for every read/write).
 ///
 /// The `KeyMgr` accessors - currently just [`get()`](KeyMgr::get) -
-/// search the configured key stores in order: first the default key store,
+/// search the configured key stores in order: first the primary key store,
 /// and then the secondary stores, in order.
 ///
 ///
@@ -39,8 +39,8 @@ use tor_error::{bad_api_usage, internal};
 #[derive(derive_builder::Builder)]
 #[builder(pattern = "owned", build_fn(private, name = "build_unvalidated"))]
 pub struct KeyMgr {
-    /// The default key store.
-    default_store: BoxedKeystore,
+    /// The primary key store.
+    primary_store: BoxedKeystore,
     /// The secondary key stores.
     #[builder(default, setter(custom))]
     secondary_stores: Vec<BoxedKeystore>,
@@ -129,7 +129,15 @@ impl KeyMgr {
     ///
     /// Returns `Ok(None)` if none of the key stores have the requested key.
     pub fn get<K: ToEncodableKey>(&self, key_spec: &dyn KeySpecifier) -> Result<Option<K>> {
-        self.get_from_store(key_spec, &K::Key::key_type(), self.all_stores())
+        let result = self.get_from_store(key_spec, &K::Key::key_type(), self.all_stores())?;
+        if result.is_none() {
+            // If the key_spec is the specifier for the public part of a keypair,
+            // try getting the pair and extracting the public portion from it.
+            if let Some(key_pair_spec) = key_spec.keypair_specifier() {
+                return Ok(self.get::<K::KeyPair>(&*key_pair_spec)?.map(|k| k.into()));
+            }
+        }
+        Ok(result)
     }
 
     /// Retrieve the specified keystore entry, and try to deserialize it as `K::Key`.
@@ -218,32 +226,40 @@ impl KeyMgr {
 
     /// Insert `key` into the [`Keystore`](crate::Keystore) specified by `selector`.
     ///
+    /// If the key already exists in the specified key store, the `overwrite` flag is used to
+    /// decide whether to overwrite it with the provided key.
+    ///
     /// If this key is not already in the keystore, `None` is returned.
     ///
     /// If this key already exists in the keystore, its value is updated
     /// and the old value is returned.
     ///
-    /// Returns an error if the selected keystore is not the default keystore or one of the
+    /// Returns an error if the selected keystore is not the primary keystore or one of the
     /// configured secondary stores.
     pub fn insert<K: ToEncodableKey>(
         &self,
         key: K,
         key_spec: &dyn KeySpecifier,
         selector: KeystoreSelector,
+        overwrite: bool,
     ) -> Result<Option<K>> {
         let key = key.to_encodable_key();
         let store = self.select_keystore(&selector)?;
         let key_type = K::Key::key_type();
         let old_key: Option<K> = self.get_from_store(key_spec, &key_type, [store].into_iter())?;
-        let () = store.insert(&key, key_spec, &key_type)?;
 
-        Ok(old_key)
+        if old_key.is_some() && !overwrite {
+            Err(crate::Error::KeyAlreadyExists)
+        } else {
+            let () = store.insert(&key, key_spec, &key_type)?;
+            Ok(old_key)
+        }
     }
 
     /// Remove the key identified by `key_spec` from the [`Keystore`](crate::Keystore)
     /// specified by `selector`.
     ///
-    /// Returns an error if the selected keystore is not the default keystore or one of the
+    /// Returns an error if the selected keystore is not the primary keystore or one of the
     /// configured secondary stores.
     ///
     /// Returns the value of the removed key,
@@ -367,23 +383,23 @@ impl KeyMgr {
 
     /// Return an iterator over all configured stores.
     fn all_stores(&self) -> impl Iterator<Item = &BoxedKeystore> {
-        iter::once(&self.default_store).chain(self.secondary_stores.iter())
+        iter::once(&self.primary_store).chain(self.secondary_stores.iter())
     }
 
     /// Return the [`Keystore`](crate::Keystore) matching the specified `selector`.
     ///
-    /// Returns an error if the selected keystore is not the default keystore or one of the
+    /// Returns an error if the selected keystore is not the primary keystore or one of the
     /// configured secondary stores.
     fn select_keystore(&self, selector: &KeystoreSelector) -> Result<&BoxedKeystore> {
         match selector {
             KeystoreSelector::Id(keystore_id) => self.find_keystore(keystore_id),
-            KeystoreSelector::Default => Ok(&self.default_store),
+            KeystoreSelector::Primary => Ok(&self.primary_store),
         }
     }
 
     /// Return the [`Keystore`](crate::Keystore) with the specified `id`.
     ///
-    /// Returns an error if the specified ID is not the ID of the default keystore or
+    /// Returns an error if the specified ID is not the ID of the primary keystore or
     /// the ID of one of the configured secondary stores.
     fn find_keystore(&self, id: &KeystoreId) -> Result<&BoxedKeystore> {
         self.all_stores()
@@ -408,13 +424,13 @@ mod tests {
     #![allow(clippy::needless_pass_by_value)]
     //! <!-- @@ end test lint list maintained by maint/add_warning @@ -->
     use super::*;
-    use crate::keystore::Sealed;
-    use crate::{ArtiPath, ArtiPathUnavailableError, ErasedKey, KeyPath, KeyType, SshKeyData};
+    use crate::{ArtiPath, ArtiPathUnavailableError, KeyPath};
     use std::collections::HashMap;
     use std::result::Result as StdResult;
     use std::str::FromStr;
     use std::sync::RwLock;
     use tor_basic_utils::test_rng::testing_rng;
+    use tor_key_forge::{EncodableKey, ErasedKey, SshKeyData};
     use tor_llcrypto::pk::ed25519;
 
     /// The type of "key" stored in the test key stores.
@@ -433,6 +449,12 @@ mod tests {
         key: SshKeyData,
     }
 
+    impl From<TestKey> for TestPublicKey {
+        fn from(tk: TestKey) -> TestPublicKey {
+            TestPublicKey { key: tk.key }
+        }
+    }
+
     impl TestKey {
         /// Create a new test key with the specified metadata.
         fn new(meta: &str) -> Self {
@@ -447,7 +469,7 @@ mod tests {
     }
 
     impl Keygen for TestKey {
-        fn generate(mut rng: &mut dyn KeygenRng) -> Result<Self>
+        fn generate(mut rng: &mut dyn KeygenRng) -> tor_key_forge::Result<Self>
         where
             Self: Sized,
         {
@@ -458,8 +480,6 @@ mod tests {
         }
     }
 
-    impl Sealed for TestKey {}
-
     impl EncodableKey for TestKey {
         fn key_type() -> KeyType
         where
@@ -469,13 +489,14 @@ mod tests {
             KeyType::Ed25519Keypair
         }
 
-        fn as_ssh_key_data(&self) -> Result<SshKeyData> {
+        fn as_ssh_key_data(&self) -> tor_key_forge::Result<SshKeyData> {
             Ok(self.key.clone())
         }
     }
 
     impl ToEncodableKey for TestKey {
         type Key = Self;
+        type KeyPair = Self;
 
         fn to_encodable_key(self) -> Self::Key {
             self
@@ -486,8 +507,6 @@ mod tests {
         }
     }
 
-    impl Sealed for TestPublicKey {}
-
     impl EncodableKey for TestPublicKey {
         fn key_type() -> KeyType
         where
@@ -496,13 +515,14 @@ mod tests {
             KeyType::Ed25519PublicKey
         }
 
-        fn as_ssh_key_data(&self) -> Result<SshKeyData> {
+        fn as_ssh_key_data(&self) -> tor_key_forge::Result<SshKeyData> {
             Ok(self.key.clone())
         }
     }
 
     impl ToEncodableKey for TestPublicKey {
         type Key = Self;
+        type KeyPair = TestKey;
 
         fn to_encodable_key(self) -> Self::Key {
             self
@@ -627,6 +647,10 @@ mod tests {
                 fn ctor_path(&self) -> Option<crate::CTorPath> {
                     None
                 }
+
+                fn keypair_specifier(&self) -> Option<Box<dyn KeySpecifier>> {
+                    None
+                }
             }
         };
     }
@@ -638,6 +662,7 @@ mod tests {
     impl_specifier!(TestKeySpecifier1, "spec1");
     impl_specifier!(TestKeySpecifier2, "spec2");
     impl_specifier!(TestKeySpecifier3, "spec3");
+    impl_specifier!(TestKeySpecifier4, "spec4");
 
     impl_specifier!(TestPublicKeySpecifier1, "pub-spec1");
 
@@ -652,7 +677,7 @@ mod tests {
 
     #[test]
     fn insert_and_get() {
-        let mut builder = KeyMgrBuilder::default().default_store(Box::<Keystore1>::default());
+        let mut builder = KeyMgrBuilder::default().primary_store(Box::<Keystore1>::default());
 
         builder
             .secondary_stores()
@@ -666,6 +691,7 @@ mod tests {
                 TestKey::new("coot"),
                 &TestKeySpecifier1,
                 KeystoreSelector::Id(&KeystoreId::from_str("keystore2").unwrap()),
+                true,
             )
             .unwrap();
 
@@ -683,6 +709,7 @@ mod tests {
                 TestKey::new("gull"),
                 &TestKeySpecifier1,
                 KeystoreSelector::Id(&KeystoreId::from_str("keystore2").unwrap()),
+                true,
             )
             .unwrap()
             .unwrap()
@@ -695,41 +722,66 @@ mod tests {
                 .map(|k| k.meta),
             Some("keystore2_gull".to_string()),
         );
-        // Insert a key into the default keystore
+
+        // Insert a different key using the _same_ key specifier (overwrite = false)
+        let err = mgr
+            .insert(
+                TestKey::new("gull"),
+                &TestKeySpecifier1,
+                KeystoreSelector::Id(&KeystoreId::from_str("keystore2").unwrap()),
+                false,
+            )
+            .unwrap_err();
+        assert!(matches!(err, crate::Error::KeyAlreadyExists));
+
+        // Insert a new key into Keystore2 (overwrite = false)
+        let old_key = mgr
+            .insert(
+                TestKey::new("penguin"),
+                &TestKeySpecifier2,
+                KeystoreSelector::Id(&KeystoreId::from_str("keystore2").unwrap()),
+                false,
+            )
+            .unwrap();
+        assert!(old_key.is_none());
+
+        // Insert a key into the primary keystore
         let old_key = mgr
             .insert(
                 TestKey::new("moorhen"),
-                &TestKeySpecifier2,
-                KeystoreSelector::Default,
+                &TestKeySpecifier3,
+                KeystoreSelector::Primary,
+                true,
             )
             .unwrap();
         assert!(old_key.is_none());
         assert_eq!(
-            mgr.get::<TestKey>(&TestKeySpecifier2)
+            mgr.get::<TestKey>(&TestKeySpecifier3)
                 .unwrap()
                 .map(|k| k.meta),
             Some("keystore1_moorhen".to_string())
         );
 
         // The key doesn't exist in any of the stores yet.
-        assert!(mgr.get::<TestKey>(&TestKeySpecifier3).unwrap().is_none());
+        assert!(mgr.get::<TestKey>(&TestKeySpecifier4).unwrap().is_none());
 
         // Insert the same key into all 3 key stores, in reverse order of keystore priority
-        // (otherwise KeyMgr::get will return the key from the default store for each iteration and
+        // (otherwise KeyMgr::get will return the key from the primary store for each iteration and
         // we won't be able to see the key was actually inserted in each store).
         for store in ["keystore3", "keystore2", "keystore1"] {
             let old_key = mgr
                 .insert(
                     TestKey::new("cormorant"),
-                    &TestKeySpecifier3,
+                    &TestKeySpecifier4,
                     KeystoreSelector::Id(&KeystoreId::from_str(store).unwrap()),
+                    true,
                 )
                 .unwrap();
             assert!(old_key.is_none());
 
             // Ensure the key now exists in `store`.
             assert_eq!(
-                mgr.get::<TestKey>(&TestKeySpecifier3)
+                mgr.get::<TestKey>(&TestKeySpecifier4)
                     .unwrap()
                     .map(|k| k.meta),
                 Some(format!("{store}_cormorant"))
@@ -739,7 +791,7 @@ mod tests {
         // The key exists in all key stores, but if no keystore_id is specified, we return the
         // value from the first key store it is found in (in this case, Keystore1)
         assert_eq!(
-            mgr.get::<TestKey>(&TestKeySpecifier3)
+            mgr.get::<TestKey>(&TestKeySpecifier4)
                 .unwrap()
                 .map(|k| k.meta),
             Some("keystore1_cormorant".to_string())
@@ -748,7 +800,7 @@ mod tests {
 
     #[test]
     fn remove() {
-        let mut builder = KeyMgrBuilder::default().default_store(Box::<Keystore1>::default());
+        let mut builder = KeyMgrBuilder::default().primary_store(Box::<Keystore1>::default());
 
         builder
             .secondary_stores()
@@ -765,6 +817,7 @@ mod tests {
             TestKey::new("coot"),
             &TestKeySpecifier1,
             KeystoreSelector::Id(&KeystoreId::from_str("keystore2").unwrap()),
+            true,
         )
         .unwrap();
         assert_eq!(
@@ -786,9 +839,9 @@ mod tests {
             .contains(&TestKeySpecifier1, &TestKey::key_type())
             .unwrap());
 
-        // Try to remove the key from the default key store
+        // Try to remove the key from the primary key store
         assert!(mgr
-            .remove::<TestKey>(&TestKeySpecifier1, KeystoreSelector::Default)
+            .remove::<TestKey>(&TestKeySpecifier1, KeystoreSelector::Primary)
             .unwrap()
             .is_none(),);
 
@@ -817,14 +870,15 @@ mod tests {
     #[test]
     fn keygen() {
         let mgr = KeyMgrBuilder::default()
-            .default_store(Box::<Keystore1>::default())
+            .primary_store(Box::<Keystore1>::default())
             .build()
             .unwrap();
 
         mgr.insert(
             TestKey::new("coot"),
             &TestKeySpecifier1,
-            KeystoreSelector::Default,
+            KeystoreSelector::Primary,
+            true,
         )
         .unwrap();
 
@@ -838,7 +892,7 @@ mod tests {
         let err = mgr
             .generate::<TestKey>(
                 &TestKeySpecifier1,
-                KeystoreSelector::Default,
+                KeystoreSelector::Primary,
                 &mut testing_rng(),
                 false,
             )
@@ -864,7 +918,7 @@ mod tests {
         let key = mgr
             .generate::<TestKey>(
                 &TestKeySpecifier1,
-                KeystoreSelector::Default,
+                KeystoreSelector::Primary,
                 &mut testing_rng(),
                 true,
             )
@@ -888,7 +942,7 @@ mod tests {
 
     #[test]
     fn get_or_generate() {
-        let mut builder = KeyMgrBuilder::default().default_store(Box::<Keystore1>::default());
+        let mut builder = KeyMgrBuilder::default().primary_store(Box::<Keystore1>::default());
 
         builder
             .secondary_stores()
@@ -904,6 +958,7 @@ mod tests {
             TestKey::new("coot"),
             &TestKeySpecifier1,
             KeystoreSelector::Id(&keystore2),
+            true,
         )
         .unwrap();
 
@@ -911,7 +966,7 @@ mod tests {
         assert_eq!(
             mgr.get_or_generate::<TestKey>(
                 &TestKeySpecifier1,
-                KeystoreSelector::Default,
+                KeystoreSelector::Primary,
                 &mut testing_rng()
             )
             .unwrap()
