@@ -55,7 +55,7 @@ pub struct TypedParticipation<T> {
 #[derive(Educe, derive_more::Display)]
 #[educe(Copy, Clone)]
 #[educe(Debug(named_field = false))]
-#[display(fmt = "{raw}")]
+#[display("{raw}")]
 pub struct TypedMemoryCost<T> {
     /// The actual cost in bytes
     raw: usize,
@@ -217,6 +217,7 @@ mod test {
     #![allow(clippy::useless_vec)]
     #![allow(clippy::needless_pass_by_value)]
     //! <!-- @@ end test lint list maintained by maint/add_warning @@ -->
+    #![allow(clippy::arithmetic_side_effects)] // don't mind potential panicking ops in tests
 
     use super::*;
     use crate::mtracker::test::*;
