@@ -2,7 +2,7 @@
 //!
 //! # Example
 //!
-//! ```
+//! ```cfg(feature = "memquota")
 //! use std::{collections::VecDeque, sync::{Arc, Mutex}};
 //! use tor_rtcompat::{CoarseInstant, CoarseTimeProvider, PreferredRuntime};
 //! use tor_memquota::{mtracker, MemoryQuotaTracker, MemoryReclaimedError, EnabledToken};
@@ -386,7 +386,7 @@ struct Global {
     total_used: TotalQtyNotifier,
 
     /// Configuration
-    config: Config,
+    config: ConfigInner,
 
     /// Make this type uninhabited if memory tracking is compiled out
     #[allow(dead_code)]
@@ -536,9 +536,10 @@ macro_rules! find_in_tracker_eh {
 
 impl MemoryQuotaTracker {
     /// Set up a new `MemoryDataTracker`
-    #[cfg(feature = "memquota")]
     pub fn new<R: Spawn>(runtime: &R, config: Config) -> Result<Arc<Self>, StartupError> {
-        let enabled = EnabledToken::new();
+        let Enabled(config, enabled) = config.0 else {
+            return Ok(MemoryQuotaTracker::new_noop());
+        };
 
         let (reclaim_tx, reclaim_rx) = mpsc::channel(0 /* plus num_senders, ie 1 */);
         let total_used = TotalQtyNotifier::new_zero(reclaim_tx);
@@ -561,6 +562,61 @@ impl MemoryQuotaTracker {
         Ok(tracker)
     }
 
+    /// Reconfigure
+    pub fn reconfigure(
+        &self,
+        new_config: Config,
+        how: tor_config::Reconfigure,
+    ) -> Result<(), ReconfigureError> {
+        use tor_config::Reconfigure;
+
+        let state = self.lock().map_err(into_internal!(
+            "cannot reconfigure corrupted memquota tracker"
+        ))?;
+
+        let (state, new_config) = match (state, new_config.0) {
+            (Noop, Noop) => return Ok(()),
+            (Noop, Enabled(..)) => return how.cannot_change(
+                // TODO #1577 (3) this isn't the `field` wanted by `cannot_change`
+ "tor-memquota max (`system.memory.max`) cannot be set: cannot enable memory quota tracking, when disabled at program start"
+            ),
+            (Enabled(state, _enabled), new_config) => {
+                let new_config = new_config.into_enabled().unwrap_or(
+                    // If the new configuration is "Noop", set the limit values to MAX
+                    // so we will never think we want to reclaim.
+                    // We don't replace ourselves with a Noop or something,
+                    // in case the user wants to re-enable tracking.
+                    ConfigInner {
+                        max: Qty::MAX,
+                        low_water: Qty::MAX,
+                    },
+                );
+
+                (state, new_config)
+            },
+        };
+
+        // Bind state mutably only if we're supposed to actually be modifying anything
+        let mut state = match how {
+            Reconfigure::CheckAllOrNothing => return Ok(()),
+            Reconfigure::AllOrNothing | Reconfigure::WarnOnFailures => state,
+            _ => Err(internal!("Reconfigure variant unknown! {how:?}"))?, // TODO #1577 (1)
+        };
+
+        let global = &mut state.global;
+        global.config = new_config;
+
+        // If the new limit is lower, we might need to start reclaiming:
+        global.total_used.maybe_wakeup(&global.config);
+
+        // If the new low_water is higher, we might need to *stop* reclaiming.
+        // We don't have a way to abort an ongoing reclaim request,
+        // but the usage vs low_water will be rechecked before we reclaim
+        // from another Participant, which will be sufficient.
+
+        Ok(())
+    }
+
     /// Returns an estimate of the total memory use
     ///
     /// The returned value is:
@@ -1036,7 +1092,10 @@ impl Participation {
             // While we're here, fill the cache to TARGET_CACHE_CLAIMING.
             // Cannot underflow: cache < want (since we failed at `got` earlier
             // and we've just checked want <= TARGET_CACHE_CLAIMING.
-            let want_more_cache = Qty(*TARGET_CACHE_CLAIMING - *self_.cache.as_raw());
+            let want_more_cache = TARGET_CACHE_CLAIMING
+                .checked_sub(*self_.cache.as_raw())
+                .expect("but cache < want");
+            let want_more_cache = Qty(want_more_cache);
             if let Ok(add_cache) = claim(want_more_cache) {
                 // On error, just don't do this; presumably the error will show up later
                 // (we mustn't early exit here, because we've got the claim in our hand).
@@ -1070,7 +1129,12 @@ impl Participation {
                     *self_.pid => precord;
                     ?None
                 }
-                let return_from_cache = Qty(*self_.cache.as_raw() - *TARGET_CACHE_RELEASING);
+                let return_from_cache = self_
+                    .cache
+                    .as_raw()
+                    .checked_sub(*TARGET_CACHE_RELEASING)
+                    .expect("TARGET_CACHE_RELEASING > MAX_CACHE ?!");
+                let return_from_cache = Qty(return_from_cache);
                 let from_cache = self_
                     .cache
                     .split_off(return_from_cache)
