@@ -42,13 +42,13 @@ enum PStatus {
 #[derive(Debug, derive_more::Display)]
 enum Outcome {
     /// We reached the low water mark
-    #[display(fmt = "complete")]
+    #[display("complete")]
     TargetReached,
 
     /// We didn't, but we have so many participants that that's possibly expected
     ///
     /// (Can only happen on 32-bit platforms.)
-    #[display(fmt = "{} participants, good enough - stopping", n_particips)]
+    #[display("{} participants, good enough - stopping", n_particips)]
     GoodEnough {
         /// The number of participants
         n_particips: NumParticips,
@@ -220,7 +220,13 @@ impl Reclaiming {
                 })
                 .sum::<NumParticips>();
 
-            if state.total_used.as_raw().as_usize() / n_particips < usize::from(MAX_CACHE) {
+            if state
+                .total_used
+                .as_raw()
+                .as_usize()
+                .checked_div(n_particips)
+                .is_some_and(|total_used| total_used < usize::from(MAX_CACHE))
+            {
                 // On 32-bit, this could happen due to the cache, if we have
                 // 2^32 / MAX_CACHE participants.
                 return stop(state, Outcome::GoodEnough { n_particips });
