@@ -394,7 +394,7 @@ pub struct NetParameters {
     /// If set to 1, introduction points use the INTRODUCE1 rate limiting
     /// defense when no `DosParams` are sent.
     ///
-    /// See <https://spec.torproject.org/param-spec.md#HiddenServiceEnableIntroDoSDefense>
+    /// See <https://spec.torproject.org/param-spec.html#HiddenServiceEnableIntroDoSDefense>
     pub hs_intro_dos_enabled: BoundedInt32<0, 1> = (0)
         from "HiddenServiceEnableIntroDoSDefense",
 
@@ -402,7 +402,7 @@ pub struct NetParameters {
     /// limiting when no `DosParams` value is sent, in messages per second.
     ///
     /// See
-    /// <https://spec.torproject.org/param-spec.md#HiddenServiceEnableIntroDoSBurstPerSec>
+    /// <https://spec.torproject.org/param-spec.html#HiddenServiceEnableIntroDoSBurstPerSec>
     pub hs_intro_dos_max_burst: BoundedInt32<0, {i32::MAX}> = (200)
         from "HiddenServiceEnableIntroDoSBurstPerSec",
 
@@ -410,9 +410,79 @@ pub struct NetParameters {
     /// limiting when no `DosParams` value is sent.
     ///
     /// See
-    /// <https://spec.torproject.org/param-spec.md#HiddenServiceEnableIntroDoSRatePerSec>
+    /// <https://spec.torproject.org/param-spec.html#HiddenServiceEnableIntroDoSRatePerSec>
     pub hs_intro_dos_rate: BoundedInt32<0, {i32::MAX}> = (25)
         from  "HiddenServiceEnableIntroDoSRatePerSec",
+
+    /// The type of vanguards to use by default when building onion service circuits:
+    ///
+    /// ```text
+    ///    0: No vanguards.
+    ///    1: Lite vanguards.
+    ///    2: Full vanguards.
+    /// ```
+    ///
+    /// See
+    /// <https://spec.torproject.org/param-spec.html#vanguards>
+    pub vanguards_enabled: BoundedInt32<0, 2> = (1)
+        from "vanguards-enabled",
+
+    /// If higher than `vanguards-enabled`,
+    /// and we are running an onion service,
+    /// we use this level for all our onion service circuits:
+    ///
+    /// ```text
+    ///    0: No vanguards.
+    ///    1: Lite vanguards.
+    ///    2: Full vanguards.
+    /// ```
+    ///
+    /// See
+    /// <https://spec.torproject.org/param-spec.html#vanguards>
+    pub vanguards_hs_service: BoundedInt32<0, 2> = (2)
+        from "vanguards-hs-service",
+
+    /// The number of vanguards in the L2 vanguard set.
+    ///
+    /// See
+    /// <https://spec.torproject.org/param-spec.html#vanguards>
+    pub guard_hs_l2_number: BoundedInt32<1, {i32::MAX}> = (4)
+        from  "guard-hs-l2-number",
+
+    /// The minimum lifetime of L2 vanguards.
+    ///
+    /// See
+    /// <https://spec.torproject.org/param-spec.html#vanguards>
+    pub guard_hs_l2_lifetime_min: IntegerSeconds<BoundedInt32<1, {i32::MAX}>> = (86400)
+        from  "guard-hs-l2-lifetime-min",
+
+    /// The maximum lifetime of L2 vanguards.
+    ///
+    /// See
+    /// <https://spec.torproject.org/param-spec.html#vanguards>
+    pub guard_hs_l2_lifetime_max: IntegerSeconds<BoundedInt32<1, {i32::MAX}>> = (1036800)
+        from  "guard-hs-l2-lifetime-max",
+
+    /// The number of vanguards in the L3 vanguard set.
+    ///
+    /// See
+    /// <https://spec.torproject.org/param-spec.html#vanguards>
+    pub guard_hs_l3_number: BoundedInt32<1, {i32::MAX}> = (8)
+        from  "guard-hs-l3-number",
+
+    /// The minimum lifetime of L3 vanguards.
+    ///
+    /// See
+    /// <https://spec.torproject.org/param-spec.html#vanguards>
+    pub guard_hs_l3_lifetime_min: IntegerSeconds<BoundedInt32<1, {i32::MAX}>> = (3600)
+        from  "guard-hs-l3-lifetime-min",
+
+    /// The maximum lifetime of L3 vanguards.
+    ///
+    /// See
+    /// <https://spec.torproject.org/param-spec.html#vanguards>
+    pub guard_hs_l3_lifetime_max: IntegerSeconds<BoundedInt32<1, {i32::MAX}>> = (172800)
+        from  "guard-hs-l3-lifetime-max",
 }
 
 }
@@ -471,6 +541,7 @@ mod test {
     #![allow(clippy::bool_assert_comparison)]
     #![allow(clippy::clone_on_copy)]
     #![allow(clippy::dbg_macro)]
+    #![allow(clippy::mixed_attributes_style)]
     #![allow(clippy::print_stderr)]
     #![allow(clippy::print_stdout)]
     #![allow(clippy::single_char_pattern)]
