@@ -9,8 +9,7 @@
 //! objects corresponding to directory requests.
 //!
 //! In general, you shouldn't try to manage channels on your own;
-//! however, there is no alternative in Arti today.  (A future
-//! channel-manager library will probably fix that.)
+//! use the `tor-chanmgr` crate instead.
 //!
 //! To launch a channel:
 //!
@@ -100,7 +99,7 @@ use testing_exports::*;
 use asynchronous_codec as futures_codec;
 use futures::channel::mpsc;
 use futures::io::{AsyncRead, AsyncWrite};
-use tor_async_utils::oneshot;
+use oneshot_fused_workaround as oneshot;
 
 use educe::Educe;
 use futures::{FutureExt as _, Sink};
