@@ -6,6 +6,7 @@
 
 use crate::err;
 use crate::err::PtError;
+use crate::PtClientMethod;
 use futures::channel::mpsc;
 use futures::channel::mpsc::Receiver;
 use futures::StreamExt;
@@ -38,8 +39,6 @@ const PT_STDIO_BUFFER: usize = 64;
 /// An arbitrary key/value status update from a pluggable transport.
 #[derive(PartialEq, Eq, Debug, Clone)]
 pub struct PtStatus {
-    /// The name of the pluggable transport.
-    transport: PtTransportName,
     /// Arbitrary key-value data about the state of this transport, from the binary running
     /// said transport.
     // NOTE(eta): This is assumed to not have duplicate keys.
@@ -345,15 +344,7 @@ impl FromStr for PtMessage {
                         message = &message[1..];
                     }
                 }
-                let transport = ret
-                    .remove("TRANSPORT")
-                    .ok_or_else(|| Cow::from("no TRANSPORT in STATUS"))?;
-                Self::Status(PtStatus {
-                    transport: transport
-                        .parse()
-                        .map_err(|_| Cow::from("bad transport ID"))?,
-                    data: ret,
-                })
+                Self::Status(PtStatus { data: ret })
             }
             _ => Self::Unknown(s.into()),
         })
@@ -581,7 +572,7 @@ pub(crate) mod sealed {
                     error: Arc::new(e),
                 })?;
 
-            let identifier = crate::pt_identifier(binary_path)?;
+            let identifier = crate::managed::pt_identifier(binary_path)?;
             AsyncPtChild::new(child, identifier)
         }
 
@@ -801,27 +792,6 @@ impl PtServerParameters {
     }
 }
 
-/// A SOCKS endpoint to connect through a pluggable transport.
-#[derive(Debug, Clone, PartialEq, Eq)]
-pub struct PtClientMethod {
-    /// The SOCKS protocol version to use.
-    pub(crate) kind: SocksVersion,
-    /// The socket address to connect to.
-    pub(crate) endpoint: SocketAddr,
-}
-
-impl PtClientMethod {
-    /// Get the SOCKS protocol version to use.
-    pub fn kind(&self) -> SocksVersion {
-        self.kind
-    }
-
-    /// Get the socket address to connect to.
-    pub fn endpoint(&self) -> SocketAddr {
-        self.endpoint
-    }
-}
-
 /// Common functionality implemented to allow code reuse
 #[async_trait::async_trait]
 #[cfg_attr(feature = "experimental-api", visibility::make(pub))]
@@ -1307,21 +1277,27 @@ mod test {
         map.insert("ADDRESS".to_string(), "198.51.100.123:1234".to_string());
         map.insert("CONNECT".to_string(), "Success".to_string());
         assert_eq!(
-            "STATUS TRANSPORT=obfs4 ADDRESS=198.51.100.123:1234 CONNECT=Success".parse(),
-            Ok(PtMessage::Status(PtStatus {
-                transport: "obfs4".parse().unwrap(),
-                data: map
-            }))
+            "STATUS ADDRESS=198.51.100.123:1234 CONNECT=Success".parse(),
+            Ok(PtMessage::Status(PtStatus { data: map }))
         );
+
+        let mut map = HashMap::new();
+        map.insert("ADDRESS".to_string(), "198.51.100.123:1234".to_string());
+        map.insert("CONNECT".to_string(), "Success".to_string());
+        map.insert("TRANSPORT".to_string(), "obfs4".to_string());
+        assert_eq!(
+            "STATUS TRANSPORT=obfs4 ADDRESS=198.51.100.123:1234 CONNECT=Success".parse(),
+            Ok(PtMessage::Status(PtStatus { data: map }))
+        );
+
         let mut map = HashMap::new();
         map.insert("ADDRESS".to_string(), "198.51.100.222:2222".to_string());
         map.insert("CONNECT".to_string(), "Failed".to_string());
         map.insert("FINGERPRINT".to_string(), "<Fingerprint>".to_string());
         map.insert("ERRSTR".to_string(), "Connection refused".to_string());
         assert_eq!(
-            "STATUS TRANSPORT=obfs4 ADDRESS=198.51.100.222:2222 CONNECT=Failed FINGERPRINT=<Fingerprint> ERRSTR=\"Connection refused\"".parse(),
+            "STATUS ADDRESS=198.51.100.222:2222 CONNECT=Failed FINGERPRINT=<Fingerprint> ERRSTR=\"Connection refused\"".parse(),
             Ok(PtMessage::Status(PtStatus {
-                transport: "obfs4".parse().unwrap(),
                 data: map
             }))
         );
