@@ -60,6 +60,11 @@ pub use obj::{Object, ObjectArcExt, ObjectId};
 #[cfg_attr(docsrs, doc(cfg(feature = "describe-methods")))]
 pub use dispatch::description::RpcDispatchInformation;
 
+#[cfg(feature = "describe-methods")]
+#[cfg_attr(docsrs, doc(cfg(feature = "describe-methods")))]
+#[doc(hidden)]
+pub use dispatch::description::DelegationNote;
+
 #[doc(hidden)]
 pub use obj::cast::CastTable;
 #[doc(hidden)]
@@ -249,7 +254,7 @@ pub const NIL: Nil = Nil {};
 /// Common return type for RPC methods that return a single object ID
 /// and nothing else.
 #[derive(Debug, Clone, serde::Serialize, serde::Deserialize, derive_more::From)]
-pub struct SingletonId {
+pub struct SingleIdResponse {
     /// The ID of the object that we're returning.
     id: ObjectId,
 }
