@@ -39,8 +39,15 @@ pub struct TokioNativeTlsRuntime {
 
 /// Implementation type for a TokioRuntimeHandle.
 #[cfg(feature = "native-tls")]
-type HandleInner =
-    CompoundRuntime<Handle, Handle, RealCoarseTimeProvider, Handle, NativeTlsProvider, Handle>;
+type HandleInner = CompoundRuntime<
+    Handle,
+    Handle,
+    RealCoarseTimeProvider,
+    Handle,
+    Handle,
+    NativeTlsProvider,
+    Handle,
+>;
 
 /// A [`Runtime`](crate::Runtime) built around a Handle to a tokio runtime, and `rustls`.
 #[derive(Clone)]
@@ -53,7 +60,7 @@ pub struct TokioRustlsRuntime {
 /// Implementation for a TokioRuntimeRustlsHandle
 #[cfg(feature = "rustls")]
 type RustlsHandleInner =
-    CompoundRuntime<Handle, Handle, RealCoarseTimeProvider, Handle, RustlsProvider, Handle>;
+    CompoundRuntime<Handle, Handle, RealCoarseTimeProvider, Handle, Handle, RustlsProvider, Handle>;
 
 #[cfg(feature = "native-tls")]
 crate::opaque::implement_opaque_runtime! {
@@ -75,6 +82,7 @@ impl From<tokio_crate::runtime::Handle> for TokioNativeTlsRuntime {
                 h.clone(),
                 RealCoarseTimeProvider::new(),
                 h.clone(),
+                h.clone(),
                 NativeTlsProvider::default(),
                 h,
             ),
@@ -92,6 +100,7 @@ impl From<tokio_crate::runtime::Handle> for TokioRustlsRuntime {
                 h.clone(),
                 RealCoarseTimeProvider::new(),
                 h.clone(),
+                h.clone(),
                 RustlsProvider::default(),
                 h,
             ),
@@ -115,6 +124,7 @@ impl TokioNativeTlsRuntime {
                 r.clone(),
                 RealCoarseTimeProvider::new(),
                 r.clone(),
+                r.clone(),
                 NativeTlsProvider::default(),
                 r,
             ),
@@ -174,6 +184,7 @@ impl TokioRustlsRuntime {
                 r.clone(),
                 RealCoarseTimeProvider::new(),
                 r.clone(),
+                r.clone(),
                 RustlsProvider::default(),
                 r,
             ),
